#include "theBiasPlanet/coreUtilities/stringsHandling/StringHandler.hpp"
#include <iomanip>
#include <iostream>
#include <locale>
#include "theBiasPlanet/coreUtilities/constantsGroups/GeneralConstantsConstantsGroup.hpp"
#include "theBiasPlanet/coreUtilities/constantsGroups/RegularExpressionsConstantsGroup.hpp"

using namespace ::std;
using namespace ::theBiasPlanet::coreUtilities::constantsGroups;

namespace theBiasPlanet {
	namespace coreUtilities {
		namespace stringsHandling {
			StringHandler::ParsedUrl::ParsedUrl (string const & a_url) {
				string::const_iterator l_urlIteratorAtStart (a_url.cbegin ());
				string::const_iterator l_urlIteratorAtEnd (a_url.cend ());
				smatch l_matcher;
				if (regex_search (l_urlIteratorAtStart, l_urlIteratorAtEnd, l_matcher, RegularExpressionsConstantsGroup::c_urlRegularExpression)) {
					i_protocol = l_matcher [2];
					string const l_nonDecodedLocalPath (l_matcher [3]);
					int const l_nonDecodedLocalPathLength = l_nonDecodedLocalPath.length ();
					int l_characterCode;
					for (int l_characterIndex = 0; l_characterIndex < l_nonDecodedLocalPathLength; l_characterIndex ++) {
						if (l_nonDecodedLocalPath [l_characterIndex] != '%') {
							if (l_nonDecodedLocalPath [l_characterIndex] == '+') {
								i_localPath += ' ';
							}
							else {
								i_localPath += l_nonDecodedLocalPath [l_characterIndex];
							}
						}
						else {
							sscanf (l_nonDecodedLocalPath.substr (l_characterIndex + 1, 2).c_str (), "%x", &l_characterCode);
							i_localPath += (char) l_characterCode;
							l_characterIndex += 2;
						}
					}
				}
			}
			
			StringHandler::ParsedUrl::~ParsedUrl () {
			}
			
			string const & StringHandler::ParsedUrl::getProtocol () {
				return i_protocol;
			}
			
			string const & StringHandler::ParsedUrl::getLocalPath () {
				return i_localPath;
			}
			
			u16string StringHandler::getUtf16String (string const & a_utf8String) {
#ifdef GCC
				return c_wstringConverter.from_bytes (a_utf8String.data ());
#else
				wstring l_wstring = c_wstringConverter.from_bytes (a_utf8String.data ());
				return u16string (l_wstring.begin (),  l_wstring.end ());
#endif
			}
			
			string StringHandler::getUtf8String (u16string const & a_utf16String) {
#ifdef GCC
				return c_wstringConverter.to_bytes (a_utf16String.data ());
#else
				return c_wstringConverter.to_bytes (wstring (a_utf16String.begin (), a_utf16String.end ()).data ());
#endif
			}
			
			string StringHandler::getString (unsigned char const * a_charactersArray, int const & a_charactersArrayLength) {
				ostringstream l_stringOutputStream;
				for (int l_characterIndex = GeneralConstantsConstantsGroup::c_iterationStartNumber; l_characterIndex < a_charactersArrayLength; l_characterIndex ++) {
					if (l_characterIndex != GeneralConstantsConstantsGroup::c_iterationStartNumber) {
						l_stringOutputStream << ", ";
					}
					l_stringOutputStream << dec << l_characterIndex << "-" <<  "0x" << uppercase << setfill ('0') << setw (2) << hex << (int) a_charactersArray [l_characterIndex];
				}
				return l_stringOutputStream.str ();
			}
			
			string StringHandler::getString (list <string> const & a_object, optional <string> const & a_delimiter) {
				ostringstream l_stringOutputStream;
				bool l_is1stElement = true;
				for (string const & l_element: a_object) {
					if (l_is1stElement) {
						l_is1stElement = false;
					}
					else {
						if (a_delimiter.has_value ()) {
							l_stringOutputStream << *a_delimiter;
						}
					}
					l_stringOutputStream << l_element;
				}
				return l_stringOutputStream.str ();
			}
			
			ostringstream * StringHandler::format (ostringstream * a_resultStream, istringstream * a_formatStream) {
				if (a_formatStream != nullptr) {
					string l_remains;
					getline (*a_formatStream, l_remains);
					(*a_resultStream) << l_remains;
				}
				else {
				}
				return a_resultStream;
			}
			
			string & StringHandler::replaceAll (string * a_targetString, regex const & a_stringFragmentToBeReplacedRegularExpression, string const & a_stringFragmentThatReplaces) {
				string l_workingString;
				l_workingString.reserve (a_targetString->length ());
				int l_currentCharacterIndex = GeneralConstantsConstantsGroup::c_iterationStartNumber;
				int l_stringFragmentToBeReplacedIndex = GeneralConstantsConstantsGroup::c_unspecifiedInteger;
				smatch l_match;
				string::const_iterator l_targetStringIteratorAtStart (a_targetString->cbegin ());
				string::const_iterator l_targetStringIteratorAtCurrentCharacter (l_targetStringIteratorAtStart);
				string::const_iterator l_targetStringIteratorAtEnd (a_targetString->cend ());
				while (regex_search (l_targetStringIteratorAtCurrentCharacter, l_targetStringIteratorAtEnd, l_match, a_stringFragmentToBeReplacedRegularExpression)) {
					l_currentCharacterIndex = distance (l_targetStringIteratorAtStart, l_targetStringIteratorAtCurrentCharacter);
					l_stringFragmentToBeReplacedIndex = l_currentCharacterIndex + l_match.position (GeneralConstantsConstantsGroup::c_iterationStartNumber);
					l_workingString.append (*a_targetString, l_currentCharacterIndex, l_stringFragmentToBeReplacedIndex - 1 - (l_currentCharacterIndex - 1));
					l_workingString += a_stringFragmentThatReplaces;
					l_targetStringIteratorAtCurrentCharacter = l_match.suffix ().first;
				}
				l_currentCharacterIndex = distance (l_targetStringIteratorAtStart, l_targetStringIteratorAtCurrentCharacter);
				l_workingString += a_targetString->substr (l_currentCharacterIndex);
				a_targetString->swap (l_workingString);
				return *a_targetString;
			}
			
			string StringHandler::getThousandsSeparatedLongString (long const & a_long) {
				ostringstream l_resultStream;
				l_resultStream.imbue (locale (""));
				l_resultStream << a_long;
				return l_resultStream.str ();
			}
			
			string StringHandler::getDateAndTimeString (time_t const & a_dateAndTime) {
    			tm const l_localDateAndTime = *(localtime (&a_dateAndTime));
    			ostringstream l_dateAndTimeExpressionStream;
    			l_dateAndTimeExpressionStream << put_time (&l_localDateAndTime, "%Y-%m-%dT%H:%M:%S");
				return l_dateAndTimeExpressionStream.str ();
			}
			
			string StringHandler::createUnNullifiedString (char const * const a_originalCharactersArray) {
				if (a_originalCharactersArray != nullptr) {
					return string (a_originalCharactersArray);
				}
				else {
					return GeneralConstantsConstantsGroup::c_emptyString;
				}
			}
			
			string StringHandler::effectuateEnvironmentVariables (string const & a_originalString) {
				string l_targetString;
				sregex_iterator l_environmentVariableMatchersIterator (a_originalString.begin (), a_originalString.end (), c_environmentVariableRegularExpression);
				smatch l_environmentVariableMatcher;
				sregex_iterator const l_environmentVariableMatchersIteratorEnd;
				for (; l_environmentVariableMatchersIterator != l_environmentVariableMatchersIteratorEnd; l_environmentVariableMatchersIterator ++) {
					l_environmentVariableMatcher = *l_environmentVariableMatchersIterator;
					l_targetString.append (l_environmentVariableMatcher.prefix ().str ());
					l_targetString.append (createUnNullifiedString (getenv (l_environmentVariableMatcher [GeneralConstantsConstantsGroup::c_iterationStartNumber + 1].str ().c_str ())));
				}
				if (l_environmentVariableMatcher.ready ()) {
					l_targetString.append (l_environmentVariableMatcher.suffix ().str ());
				}
				else {
					l_targetString.append (a_originalString);
				}
				return l_targetString;
			}
			
			bool StringHandler::validateAsUrl (string const & a_string) {
				smatch l_match;
				string::const_iterator l_stringIteratorAtStart (a_string.cbegin ());
				string::const_iterator l_stringIteratorAtCurrentCharacter (l_stringIteratorAtStart);
				string::const_iterator l_stringIteratorAtEnd (a_string.cend ());
				return (regex_search (l_stringIteratorAtCurrentCharacter, l_stringIteratorAtEnd, l_match, RegularExpressionsConstantsGroup::c_urlRegularExpression));
			}
		}
	}
}

