namespace theBiasPlanet {
	namespace filesConverter {
		namespace programs {
			using System;
			using System.Collections.Generic;
			using System.IO;
			using System.Text;
			using System.Threading;
			using uno;
			using unoidl.com.sun.star.beans;
			using theBiasPlanet.coreUtilities.collectionsHandling;
			using theBiasPlanet.coreUtilities.constantsGroups;
			using theBiasPlanet.coreUtilities.cryptographyHandling;
			using theBiasPlanet.coreUtilities.inputsHandling;
			using theBiasPlanet.coreUtilities.jsonDataHandling;
			using theBiasPlanet.coreUtilities.messagingHandling;
			using theBiasPlanet.coreUtilities.processesHandling;
			using theBiasPlanet.coreUtilities.performanceMeasuringHandling;
			using theBiasPlanet.coreUtilities.pipes;
			using theBiasPlanet.coreUtilities.stringsHandling;
			using theBiasPlanet.unoUtilities.connectionsHandling;
			using theBiasPlanet.unoUtilities.constantsGroups;
			using theBiasPlanet.unoUtilities.constantsGroups.propertyNamesSets;
			using theBiasPlanet.unoUtilities.cryptographyHandling;
			using theBiasPlanet.unoUtilities.connection;
			using theBiasPlanet.unoUtilities.displayElements;
			using theBiasPlanet.unoUtilities.documentsHandling;
			using theBiasPlanet.unoUtilities.fileConversionsHandling;
			using theBiasPlanet.unoUtilities.nameValuePairsHandling;
			using theBiasPlanet.unoUtilities.pointers;
			using theBiasPlanet.unoUtilities.processesHandling;
			using theBiasPlanet.unoUtilities.propertiesHandling;
			using theBiasPlanet.unoUtilities.unoDataHandling;
			using theBiasPlanet.unoUtilities.unoObjectsContexts;
			
			public class FilesConverterConsoleProgram {
				private static readonly Int32 c_conversionsOrderFilePathsStartIndexInArgumentsArray;
				public class FileConversionOrderExtendedJsonDatumParseEventsHandler : ExtendedJsonDatumParseEventsHandler {
					static private readonly String c_disabledKey;
					static private readonly String c_originalFileUrlKey;
					static private readonly String c_targetFileUrlKey;
					static private readonly String c_fileStoringFilterNameKey;
					static private readonly String c_openingPasswordKey;
					static private readonly String c_titleKey;
					static private readonly String c_encryptingPasswordKey;
					static private readonly String c_editingPasswordKey;
					static private readonly String c_certificateIssuerNameKey;
					static private readonly String c_certificatePasswordKey;
					static private readonly String c_signatureTimeStampAuthorityUrlKey;
					static private readonly String c_unoDocumentTailorNamesKey;
					static private readonly String c_allSpreadSheetsAreExportedKey;
					static private readonly String c_targetFileNamingRuleKey;
					static private readonly String c_csvItemsDelimiterCharacterCodeKey;
					static private readonly String c_csvTextItemQuotationCharacterCodeKey;
					static private readonly String c_csvCharactersEncodingCodeKey;
					static private readonly String c_csvAllTextItemsAreQuotedKey;
					static private readonly String c_csvContentsAreExportedAsShownKey;
					static private readonly String c_csvFormulaeThemselvesAreExportedKey;
					static private readonly String c_hiddenSpreadSheetsAreExportedKey;
					private String i_currentKey;
					private Boolean i_disabled;
					private String i_originalFileUrl;
					private String i_targetFileUrl;
					private String i_fileStoringFilterName;
					private String i_openingPassword;
					private String i_title;
					private String i_encryptingPassword;
					private String i_editingPassword;
					private String i_certificateIssuerName;
					private String i_certificatePassword;
					private	String i_signatureTimeStampAuthorityUrl;
					private	List <String> i_unoDocumentTailorNames;
					private Boolean i_allSpreadSheetsAreExported;
					private	Int32 i_targetFileNamingRule;
					private Int32 i_csvItemsDelimiterCharacterCode;
					private Int32 i_csvTextItemQuotationCharacterCode;
					private Int32 i_csvCharactersEncodingCode;
					private Boolean i_csvAllTextItemsAreQuoted;
					private Boolean i_csvContentsAreExportedAsShown;
					private Boolean i_csvFormulaeThemselvesAreExported;
					private Boolean i_hiddenSpreadSheetsAreExported;
					
					static FileConversionOrderExtendedJsonDatumParseEventsHandler () {
						c_disabledKey = "Disabled";
						c_originalFileUrlKey = "OriginalFileUrl";
						c_targetFileUrlKey = "TargetFileUrl";
						c_fileStoringFilterNameKey = "FileStoringFilterName";
						c_openingPasswordKey = "OpeningPassword";
						c_titleKey = "Title";
						c_encryptingPasswordKey = "EncryptingPassword";
						c_editingPasswordKey = "EditingPassword";
						c_certificateIssuerNameKey = "CertificateIssuerName";
						c_certificatePasswordKey = "CertificatePassword";
						c_signatureTimeStampAuthorityUrlKey = "SignatureTimeStampAuthorityUrl";
						c_unoDocumentTailorNamesKey = "DocumentTailorNames";
						c_allSpreadSheetsAreExportedKey = "AllSpreadSheetsAreExported";
						c_targetFileNamingRuleKey = "TargetFileNamingRule";
						c_csvItemsDelimiterCharacterCodeKey = "CsvItemsDelimiterCharacterCode";
						c_csvTextItemQuotationCharacterCodeKey = "CsvTextItemQuotationCharacterCode";
						c_csvCharactersEncodingCodeKey = "CsvCharactersEncodingCode";
						c_csvAllTextItemsAreQuotedKey = "CsvAllTextItemsAreQuoted";
						c_csvContentsAreExportedAsShownKey = "CsvContentsAreExportedAsShown";
						c_csvFormulaeThemselvesAreExportedKey = "CsvFormulaeThemselvesAreExported";
						c_hiddenSpreadSheetsAreExportedKey = "HiddenSpreadSheetsAreExported";
					}
					
					public override void initialize () {
						i_currentKey = null;
						i_disabled = false;
						i_originalFileUrl = null;
						i_targetFileUrl = null;
						i_fileStoringFilterName = null;
						i_openingPassword = null;
						i_title = null;
						i_encryptingPassword = null;
						i_editingPassword = null;
						i_certificateIssuerName = null;
						i_certificatePassword = null;
						i_signatureTimeStampAuthorityUrl = null;
						i_unoDocumentTailorNames = null;
						i_allSpreadSheetsAreExported = false;
						i_targetFileNamingRule = 0;
						i_csvItemsDelimiterCharacterCode = (Int32) ',';
						i_csvTextItemQuotationCharacterCode = (Int32) '\"';
						i_csvCharactersEncodingCode = UnoCharactersEncodingCodesConstantsGroup.c_utf8; // 76: UTF-8, 65535: UCS-2, 65534: UCS-4, 11: US-ASCII, 69: EUC_JP, 64: SHIFT_JIS
						i_csvAllTextItemsAreQuoted = false;
						i_csvContentsAreExportedAsShown = false;
						i_csvFormulaeThemselvesAreExported = false;
						i_hiddenSpreadSheetsAreExported = false;
					}
					
					public Boolean getDisabled () {
						return i_disabled;
					}
					
					public String getOriginalFileUrl () {
						return i_originalFileUrl;
					}
					
					public String getTargetFileUrl () {
						return i_targetFileUrl;
					}
					
					public String getFileStoringFilterName () {
						return i_fileStoringFilterName;
					}
					
					public String getOpeningPassword () {
						return i_openingPassword;
					}
					
					public String getTitle () {
						return i_title;
					}
					
					public String getEncryptingPassword () {
						return i_encryptingPassword;
					}
					
					public String getEditingPassword () {
						return i_editingPassword;
					}
					
					public String getCertificateIssuerName () {
						return i_certificateIssuerName;
					}
					
					public String getCertificatePassword () {
						return i_certificatePassword;
					}
					
					public String getSignatureTimeStampAuthorityUrl () {
						return i_signatureTimeStampAuthorityUrl;
					}
					
					public List <String> getUnoDocumentTailorNames () {
						return i_unoDocumentTailorNames;
					}
					
					public Boolean getAllSpreadSheetsAreExported () {
						return i_allSpreadSheetsAreExported;
					}
					
					public Int32 getTargetFileNamingRule () {
						return i_targetFileNamingRule;
					}
					
					public Int32 getCsvItemsDelimiterCharacterCode () {
						return i_csvItemsDelimiterCharacterCode;
					}
					
					public Int32 getCsvTextItemQuotationCharacterCode () {
						return i_csvTextItemQuotationCharacterCode;
					}
					
					public Int32 getCsvCharactersEncodingCode () {
						return i_csvCharactersEncodingCode;
					}
					
					public Boolean getCsvAllTextItemsAreQuoted () {
						return i_csvAllTextItemsAreQuoted;
					}
					
					public Boolean getCsvContentsAreExportedAsShown () {
						return i_csvContentsAreExportedAsShown;
					}
					
					public Boolean getCsvFormulaeThemselvesAreExported () {
						return i_csvFormulaeThemselvesAreExported;
					}
					
					public Boolean getHiddenSpreadSheetsAreExported () {
						return i_hiddenSpreadSheetsAreExported;
					}
					
					public override Boolean onBooleanFound (Boolean a_value) {
						if (i_currentKey == c_disabledKey) {
							 i_disabled = a_value;
							 if (i_disabled) {
								 return false;
							 }
						}
						else if (i_currentKey == c_allSpreadSheetsAreExportedKey) {
							 i_allSpreadSheetsAreExported = a_value;
						}
						else if (i_currentKey == c_csvAllTextItemsAreQuotedKey) {
							i_csvAllTextItemsAreQuoted = a_value;
						}
						else if (i_currentKey == c_csvContentsAreExportedAsShownKey) {
							i_csvContentsAreExportedAsShown = a_value;
						}
						else if (i_currentKey == c_csvFormulaeThemselvesAreExportedKey) {
							i_csvFormulaeThemselvesAreExported = a_value;
						}
						else if (i_currentKey == c_hiddenSpreadSheetsAreExportedKey) {
							i_hiddenSpreadSheetsAreExported = a_value;
						}
						i_currentKey = null;
						return true;
					}
					
					public override Boolean onIntegerFound (Int32 a_value) {
						if (i_currentKey == c_targetFileNamingRuleKey) {
							i_targetFileNamingRule = a_value;
						}
						else if (i_currentKey == c_csvItemsDelimiterCharacterCodeKey) {
							i_csvItemsDelimiterCharacterCode = a_value;
						}
						else if (i_currentKey == c_csvTextItemQuotationCharacterCodeKey) {
							i_csvTextItemQuotationCharacterCode = a_value;
						}
						else if (i_currentKey == c_csvCharactersEncodingCodeKey) {
							i_csvCharactersEncodingCode = a_value;
						}
						i_currentKey = null;
						return true;
					}
					
					public override Boolean onDoubleFound (Double a_value) {
						return true;
					}
					
					public override Boolean onLocalDateAndTimeFound (DateTime a_value) {
						return true;
					}
					
					public override Boolean onLocalDateFound (DateTime a_value) {
						return true;
					}
					
					public override Boolean onLocalTimeFound (DateTime a_value) {
						return true;
					}
					
					public override Boolean onBytesArrayFound (StringPipe a_reader) {
						return true;
					}
					
					public override Boolean onStringFound (StringPipe a_reader) {
						String l_string = a_reader.readWholeString ();
						if (i_currentKey == null) {
							i_currentKey = l_string;
							if (i_currentKey == c_unoDocumentTailorNamesKey) {
								i_unoDocumentTailorNames = new List <String> ();
							}
						}
						else {
							if (i_currentKey == c_originalFileUrlKey) {
								i_originalFileUrl = StringHandler.effectuateEnvironmentVariables (l_string);
							}
							else if (i_currentKey == c_targetFileUrlKey) {
								i_targetFileUrl = StringHandler.effectuateEnvironmentVariables (l_string);
							}
							else if (i_currentKey == c_fileStoringFilterNameKey) {
								i_fileStoringFilterName = l_string;
							}
							else if (i_currentKey == c_openingPasswordKey) {
								i_openingPassword = l_string;
							}
							else if (i_currentKey == c_titleKey) {
								i_title = l_string;
							}
							else if (i_currentKey == c_encryptingPasswordKey) {
								i_encryptingPassword = l_string;
							}
							else if (i_currentKey == c_editingPasswordKey) {
								i_editingPassword = l_string;
							}
							else if (i_currentKey == c_certificateIssuerNameKey) {
								i_certificateIssuerName = l_string;
							}
							else if (i_currentKey == c_certificatePasswordKey) {
								i_certificatePassword = l_string;
							}
							else if (i_currentKey == c_signatureTimeStampAuthorityUrlKey) {
								i_signatureTimeStampAuthorityUrl = l_string;
							}
							else if (i_currentKey == c_unoDocumentTailorNamesKey) {
								i_unoDocumentTailorNames.Add (l_string);
							}
							if (i_currentKey != c_unoDocumentTailorNamesKey) {
								i_currentKey = null;
							}
						}
						return true;
					}
					
					public override Boolean onArrayStarted () {
						return true;
					}
					
					public override Boolean onArrayEnded () {
						if (i_currentKey == c_unoDocumentTailorNamesKey) {
							i_currentKey = null;
						}
						return true;
					}
					
					public override Boolean onDictionaryStarted () {
						return true;
					}
					
					public override Boolean onDictionaryEnded () {
						return true;
					}
				}
				
				static FilesConverterConsoleProgram () {
					c_conversionsOrderFilePathsStartIndexInArgumentsArray = 1;
				}
				
				public static void Main (String [] a_argumentsArray) {
					Int32 l_resultStatus = -1;
					try {
						if (a_argumentsArray.Length < 2) {
							throw new Exception ("The arguments have to be these.\nThe argument 1: the server URL like 'socket,host=localhost,port=2002,tcpNoDelay=1;urp;StarOffice.ComponentContext'\nThe argument 2: the path of the conversion orders JSON file\n");
						}
						String l_unoServerUrl = a_argumentsArray [0];
						UnoProcessEnvironment l_localUnoProcessEnvironment = new UnoProcessEnvironment (DateTime.Now.ToString (), null);
						ProcessEnvironment.windowsSocketStartup ();
						UnoConnectionConnector l_unoConnectionConnector = new UnoConnectionConnector (l_localUnoProcessEnvironment.getLocalObjectsContext ());
						UnoConnection l_unoConnection = l_unoConnectionConnector.connect (l_unoServerUrl, null);
						if (l_unoConnection == null) {
							throw new System.Exception (String.Format ("{0:s}", "The UNO connection could not be established."));
						}
						Publisher.logNormalInformation ("### A connection has been established.");
						RemoteUnoObjectsContext l_remoteUnoObjectsContext = l_unoConnection.getRemoteUnoObjectsContext ();
						FilesConverter l_filesConverter = new FilesConverter (UnoDesktop.getInstance (l_remoteUnoObjectsContext));
						IDictionary <String, UnoDocumentTailor> l_unoDocumentTailorNameToUnoDocumentTailorMap = MapsFactory.createNavigableLinkedHashMap <String, UnoDocumentTailor> ("UnoSpreadSheetsDocumentMoveSpecifiedSheetTo1stPositionTailor_1", new UnoSpreadSheetsDocumentMoveSpecifiedSheetTo1stPositionTailor (l_remoteUnoObjectsContext, 1));
						List <Thread> l_conversionsThreads = new List <Thread> ();
						PerformanceMeasurer.setStartTime ();
						for (Int32 l_conversionOrdersFilePathIndex = GeneralConstantsConstantsGroup.c_iterationStartNumber; l_conversionOrdersFilePathIndex < a_argumentsArray.Length - c_conversionsOrderFilePathsStartIndexInArgumentsArray; l_conversionOrdersFilePathIndex ++) {
							String l_conversionOrdersFilePath = a_argumentsArray [c_conversionsOrderFilePathsStartIndexInArgumentsArray + l_conversionOrdersFilePathIndex];
							Thread l_conversionsThread = new Thread ( () => {
								try {
									executeConversionOrders (l_conversionOrdersFilePath, l_filesConverter, l_unoDocumentTailorNameToUnoDocumentTailorMap);
								}
								catch (Exception l_exception) {
									Publisher.logErrorInformation (l_exception);
								}
							});
							l_conversionsThreads.Add (l_conversionsThread);
							l_conversionsThread.Start ();
						}
						foreach (Thread l_conversionsThread in l_conversionsThreads) {
							l_conversionsThread.Join ();
						}
						Console.Out.WriteLine (String.Format ("### The elapsed time is {0:n0} ns.", PerformanceMeasurer.getElapseTimeInNanoSeconds ()));
						l_unoConnection.disconnect ();
						ProcessEnvironment.windowsSocketCleanup ();
						l_resultStatus = 0;
					}
					catch (Exception l_exception) {
						Publisher.logErrorInformation (l_exception);
						Environment.Exit (l_resultStatus);
					}
					Environment.Exit (l_resultStatus);
				}
				
				public static void executeConversionOrders (String a_conversionOrdersFilePath, FilesConverter a_filesConverter, IDictionary <String, UnoDocumentTailor> a_unoDocumentTailorNameToUnoDocumentTailorMap) {
					Int32 l_conversionOrderIndex = GeneralConstantsConstantsGroup.c_iterationStartNumber;
					String l_conversionOrder = null;
					StringBuilder l_failedConversionOrderIndicesStringBuilder = new StringBuilder ();
					Boolean l_conversionHasSucceeded = false;
					Int32 l_jsonDictionaryLineStartIndex = GeneralConstantsConstantsGroup.c_unspecifiedInteger;
					Int32 l_jsonDictionaryLineEndIndex = GeneralConstantsConstantsGroup.c_unspecifiedInteger;
					StreamReader l_conversionOrdersFileReader = new StreamReader (a_conversionOrdersFilePath);
					ExtendedJsonDatumParser l_conversionOrderExtendedJsonDatumParser = new ExtendedJsonDatumParser ();
					FileConversionOrderExtendedJsonDatumParseEventsHandler l_fileConversionOrderExtendedJsonDatumParseEventsHandler = new FileConversionOrderExtendedJsonDatumParseEventsHandler ();
					while (true) {
						l_conversionHasSucceeded = false;
						l_conversionOrder = l_conversionOrdersFileReader.ReadLine ();
						if (l_conversionOrder == null) {
							break;
						}
						else {
							if (l_conversionOrder.StartsWith (GeneralConstantsConstantsGroup.c_jsonArrayOpener.ToString ()) || l_conversionOrder.StartsWith (GeneralConstantsConstantsGroup.c_jsonArrayCloser.ToString ())) {
								l_conversionOrderIndex ++;
								continue;
							}
							l_jsonDictionaryLineStartIndex = l_conversionOrder.IndexOf (GeneralConstantsConstantsGroup.c_jsonDictionaryOpener);
							l_jsonDictionaryLineEndIndex = l_conversionOrder.LastIndexOf (GeneralConstantsConstantsGroup.c_jsonDictionaryCloser);
							if (l_jsonDictionaryLineStartIndex < GeneralConstantsConstantsGroup.c_iterationStartNumber || l_jsonDictionaryLineEndIndex < GeneralConstantsConstantsGroup.c_iterationStartNumber) {
								Publisher.logErrorInformation (String.Format ("The conversion order of the index, '{0:d}', doesn't contain any valid JSON dictionary.'\n", l_conversionOrderIndex));
							}
							else {
								l_conversionOrder = l_conversionOrder.Substring (l_jsonDictionaryLineStartIndex, l_jsonDictionaryLineEndIndex + 1);
								
								try {
									l_conversionHasSucceeded = executeConversionOrder (l_conversionOrder, a_filesConverter, a_unoDocumentTailorNameToUnoDocumentTailorMap, l_conversionOrderExtendedJsonDatumParser, l_fileConversionOrderExtendedJsonDatumParseEventsHandler);
								}
								catch (Exception l_exception) {
									Publisher.logErrorInformation (String.Format ("The order of the index, '{0:d}' has failed: {1:s}.", l_conversionOrderIndex, l_exception.ToString ()));
								}
								if (!l_conversionHasSucceeded) {
									if (l_failedConversionOrderIndicesStringBuilder.Length == 0) {
									}
									else {
										l_failedConversionOrderIndicesStringBuilder.Append (",");
									}
									l_failedConversionOrderIndicesStringBuilder.Append (String.Format ("{0:d}", l_conversionOrderIndex));
								}
							}
						}
						l_conversionOrderIndex ++;
					}
					l_conversionOrdersFileReader.Close ();
					// Foolishly, "}}" cannot be put into the format string
					Console.Out.WriteLine (String.Format ("### The failed conversion order indices of '{0:s}' are {{{1:s}{2:s}.\n", a_conversionOrdersFilePath, l_failedConversionOrderIndicesStringBuilder.ToString (), "}"));
				}
				
				public static Boolean executeConversionOrder (String a_conversionOrder, FilesConverter a_filesConverter, IDictionary <String, UnoDocumentTailor> a_unoDocumentTailorNameToUnoDocumentTailorMap, ExtendedJsonDatumParser a_conversionOrderExtendedJsonDatumParser, FileConversionOrderExtendedJsonDatumParseEventsHandler a_fileConversionOrderExtendedJsonDatumParseEventsHandler) {
					Boolean l_conversionHasSucceeded = false;
					Boolean l_overwrites = true;
					String l_originalFileUrl = null;
					String l_targetFileUrl = null;
					String l_fileStoringFilterName = null;
					String l_encryptingPassword = null;
					List <String> l_encryptionDatumNames = ListsFactory.createList <String> ("OOXPassword", "CryptoType");
					List <Any> l_encryptionDatumValues = null;
					NamedValue [] l_encryptionDatumArray = null;
					String l_editingPassword = null;
					List <String> l_editingPasswordInformationPropertyNames = ListsFactory.createList <String> ("algorithm-name", "salt", "iteration-count", "hash");
					Int32 l_saltLength = 16;
					Byte [] l_salt = Hasher.createSalt (l_saltLength);
					Int32 l_numberOfIterationForHashing = 100000;
					Int32 l_editingPasswordHashLength = 16;
					Byte [] l_editingPasswordHash = null;
					List <Any> l_editingPasswordInformationPropertyValues = null;
					PropertyValue [] l_editingPasswordInformationPropertiesArray = null;
					Int32 l_editingPasswordMicrosoftHash = GeneralConstantsConstantsGroup.c_unspecifiedInteger;
					Any l_editingPasswordInformation = Any.VOID;
					String l_certificateIssuerName = null;
					String l_signatureTimeStampAuthorityUrl = null;
					List <String> l_documentStoringFilterDataInPropertiesArrayPropertyNames = null;
					List <Any> l_documentStoringFilterDataInPropertiesArrayPropertyValues = null;
					String l_documentStoringFilterDataInStringValue = null;
					PropertyValue [] l_documentStoringPropertiesArray = null;
					List <UnoDocumentTailor> l_unoDocumentTailors = null;
					a_conversionOrderExtendedJsonDatumParser.parse (new StringReader (a_conversionOrder), a_fileConversionOrderExtendedJsonDatumParseEventsHandler);
					l_originalFileUrl = a_fileConversionOrderExtendedJsonDatumParseEventsHandler.getOriginalFileUrl ();
					l_targetFileUrl = a_fileConversionOrderExtendedJsonDatumParseEventsHandler.getTargetFileUrl ();
					l_fileStoringFilterName = a_fileConversionOrderExtendedJsonDatumParseEventsHandler.getFileStoringFilterName ();
					if (a_fileConversionOrderExtendedJsonDatumParseEventsHandler.getDisabled ()) {
						return true;
					}
					if (l_originalFileUrl == null) {
						throw new Exception ("The original file URL specification is missing.");
					}
					else if (l_targetFileUrl == null) {
						throw new Exception ("The target file URL specification is missing.");
					}
					else if (l_fileStoringFilterName == null) {
						throw new Exception ("The filter name specification is missing.");
					}
					else {
						List <String> l_unoDocumentTailorNames = a_fileConversionOrderExtendedJsonDatumParseEventsHandler.getUnoDocumentTailorNames ();
						if (l_unoDocumentTailorNames != null) {
							l_unoDocumentTailors = new List <UnoDocumentTailor> ();
							try {
								foreach (String l_unoDocumentTailorName in l_unoDocumentTailorNames) {
									l_unoDocumentTailors.Add (a_unoDocumentTailorNameToUnoDocumentTailorMap [l_unoDocumentTailorName]);
								}
							}
							catch (Exception) {
							}
						}
						if (!(l_fileStoringFilterName == UnoDocumentStoringFilterNamesConstantsGroup.c_toCsvFileFilterName)) {
							l_encryptingPassword = a_fileConversionOrderExtendedJsonDatumParseEventsHandler.getEncryptingPassword ();
							if (l_encryptingPassword != null) {
								if (l_fileStoringFilterName == UnoDocumentStoringFilterNamesConstantsGroup.c_toMicrosoftWord2007XmlFileFilterName || l_fileStoringFilterName == UnoDocumentStoringFilterNamesConstantsGroup.c_toMicrosoftExcel2007XmlFileFilterName || l_fileStoringFilterName == UnoDocumentStoringFilterNamesConstantsGroup.c_toMicrosoftPowerPoint2007XmlFileFilterName) {
									l_encryptionDatumValues = ListsFactory.createList <Any> (UnoDatumConverter.getAny (l_encryptingPassword), UnoDatumConverter.getAny ("Standard"));
									l_encryptionDatumArray = UnoNameValuePairsHandler.buildNameValuePairsArray (l_encryptionDatumNames, l_encryptionDatumValues);
									l_encryptingPassword = null;
								}
								else if (l_fileStoringFilterName == UnoDocumentStoringFilterNamesConstantsGroup.c_toPdfFileFilterNameForTextDocuments || l_fileStoringFilterName == UnoDocumentStoringFilterNamesConstantsGroup.c_toPdfFileFilterNameForSpreadSheetsDocuments || l_fileStoringFilterName == UnoDocumentStoringFilterNamesConstantsGroup.c_toPdfFileFilterNameForPresentationDocuments) {
									if (l_documentStoringFilterDataInPropertiesArrayPropertyNames == null) {
										l_documentStoringFilterDataInPropertiesArrayPropertyNames = ListsFactory.createList <String> ();
										l_documentStoringFilterDataInPropertiesArrayPropertyValues = ListsFactory.createList <Any> ();
									}
									l_documentStoringFilterDataInPropertiesArrayPropertyNames.Add (UnoPdfExportingPropertyNamesSet.c_fileIsEncrypted_boolean);
									l_documentStoringFilterDataInPropertiesArrayPropertyNames.Add (UnoPdfExportingPropertyNamesSet.c_openingPassword_string);
									l_documentStoringFilterDataInPropertiesArrayPropertyValues.Add (UnoDatumConverter.getAny (true));
									l_documentStoringFilterDataInPropertiesArrayPropertyValues.Add (UnoDatumConverter.getAny (l_encryptingPassword));
									l_encryptingPassword = null;
								}
							}
							l_editingPassword = a_fileConversionOrderExtendedJsonDatumParseEventsHandler.getEditingPassword ();
							if (l_editingPassword != null) {
								if (! (l_fileStoringFilterName == UnoDocumentStoringFilterNamesConstantsGroup.c_toMicrosoftWord2007XmlFileFilterName) && ! (l_fileStoringFilterName == UnoDocumentStoringFilterNamesConstantsGroup.c_toMicrosoftWord97FileFilterName) && ! (l_fileStoringFilterName == UnoDocumentStoringFilterNamesConstantsGroup.c_toMicrosoftExcel2007XmlFileFilterName) && ! (l_fileStoringFilterName == UnoDocumentStoringFilterNamesConstantsGroup.c_toMicrosoftExcel97FileFilterName) && ! (l_fileStoringFilterName == UnoDocumentStoringFilterNamesConstantsGroup.c_toMicrosoftPowerPoint2007XmlFileFilterName) && ! (l_fileStoringFilterName == UnoDocumentStoringFilterNamesConstantsGroup.c_toMicrosoftPowerPoint97FileFilterName)) {
									l_editingPasswordHash = Hasher.hashInPbkdf2 (l_editingPassword, l_salt, l_numberOfIterationForHashing, l_editingPasswordHashLength);
									l_editingPasswordInformationPropertyValues = ListsFactory.createList <Any> (UnoDatumConverter.getAny ("PBKDF2"), UnoDatumConverter.getAny (l_salt), UnoDatumConverter.getAny (l_numberOfIterationForHashing), UnoDatumConverter.getAny (l_editingPasswordHash));
									l_editingPasswordInformationPropertiesArray = UnoPropertiesHandler.buildPropertiesArray (l_editingPasswordInformationPropertyNames, l_editingPasswordInformationPropertyValues);
									l_editingPasswordInformation = UnoDatumConverter.getAny (l_editingPasswordInformationPropertiesArray);
								}
								else {
									if (l_fileStoringFilterName == UnoDocumentStoringFilterNamesConstantsGroup.c_toMicrosoftWord2007XmlFileFilterName || l_fileStoringFilterName == UnoDocumentStoringFilterNamesConstantsGroup.c_toMicrosoftWord97FileFilterName || l_fileStoringFilterName == UnoDocumentStoringFilterNamesConstantsGroup.c_toMicrosoftExcel2007XmlFileFilterName || l_fileStoringFilterName == UnoDocumentStoringFilterNamesConstantsGroup.c_toMicrosoftPowerPoint2007XmlFileFilterName) {
										l_editingPasswordMicrosoftHash = MicrosoftPasswordsHasher.hashIn32bits (l_editingPassword);
									}
									else {
										l_editingPasswordMicrosoftHash = MicrosoftPasswordsHasher.hashIn16bits (l_editingPassword);
									}
									l_editingPasswordInformation = UnoDatumConverter.getAny (l_editingPasswordMicrosoftHash);
								}
							}
							l_certificateIssuerName = a_fileConversionOrderExtendedJsonDatumParseEventsHandler.getCertificateIssuerName ();
							if (l_certificateIssuerName != null) {
								if (l_fileStoringFilterName == UnoDocumentStoringFilterNamesConstantsGroup.c_toPdfFileFilterNameForTextDocuments || l_fileStoringFilterName == UnoDocumentStoringFilterNamesConstantsGroup.c_toPdfFileFilterNameForSpreadSheetsDocuments || l_fileStoringFilterName == UnoDocumentStoringFilterNamesConstantsGroup.c_toPdfFileFilterNameForPresentationDocuments) {
									if (l_documentStoringFilterDataInPropertiesArrayPropertyNames == null) {
										l_documentStoringFilterDataInPropertiesArrayPropertyNames = ListsFactory.createList <String> ();
										l_documentStoringFilterDataInPropertiesArrayPropertyValues = ListsFactory.createList <Any> ();
									}
									l_documentStoringFilterDataInPropertiesArrayPropertyNames.Add (UnoPdfExportingPropertyNamesSet.c_fileIsSigned_boolean);
									l_documentStoringFilterDataInPropertiesArrayPropertyNames.Add (UnoPdfExportingPropertyNamesSet.c_signatureCertificate_com_sun_star_security_XCertificate);
									l_documentStoringFilterDataInPropertiesArrayPropertyNames.Add (UnoPdfExportingPropertyNamesSet.c_signatureCertificatePassword_string);
									l_documentStoringFilterDataInPropertiesArrayPropertyValues.Add (UnoDatumConverter.getAny (true));
									l_documentStoringFilterDataInPropertiesArrayPropertyValues.Add (UnoDatumConverter.getAny (a_filesConverter.getRemoteUnoObjectsContext ().getNetworkSecurityServicesCertificate (l_certificateIssuerName, null)));
									l_documentStoringFilterDataInPropertiesArrayPropertyValues.Add (UnoDatumConverter.getAny (a_fileConversionOrderExtendedJsonDatumParseEventsHandler.getCertificatePassword ()));
								}
							}
							l_signatureTimeStampAuthorityUrl = a_fileConversionOrderExtendedJsonDatumParseEventsHandler.getSignatureTimeStampAuthorityUrl ();
							if (l_signatureTimeStampAuthorityUrl != null) {
								if (l_fileStoringFilterName == UnoDocumentStoringFilterNamesConstantsGroup.c_toPdfFileFilterNameForTextDocuments || l_fileStoringFilterName == UnoDocumentStoringFilterNamesConstantsGroup.c_toPdfFileFilterNameForSpreadSheetsDocuments || l_fileStoringFilterName == UnoDocumentStoringFilterNamesConstantsGroup.c_toPdfFileFilterNameForPresentationDocuments) {
									if (l_documentStoringFilterDataInPropertiesArrayPropertyNames == null) {
										l_documentStoringFilterDataInPropertiesArrayPropertyNames = ListsFactory.createList <String> ();
										l_documentStoringFilterDataInPropertiesArrayPropertyValues = ListsFactory.createList <Any> ();
									}
									l_documentStoringFilterDataInPropertiesArrayPropertyNames.Add (UnoPdfExportingPropertyNamesSet.c_signatureTimeStampAuthorityUrl_string);
									l_documentStoringFilterDataInPropertiesArrayPropertyValues.Add (UnoDatumConverter.getAny (l_signatureTimeStampAuthorityUrl));
								}
							}
							List <String> l_documentStoringPropertyNames = ListsFactory.createList <String> (
								UnoDocumentStoringEnumerablePropertyNamesSet.c_filterName_string,
								UnoDocumentStoringEnumerablePropertyNamesSet.c_filterData_any,
								UnoDocumentStoringEnumerablePropertyNamesSet.c_filterData_string,
								UnoDocumentStoringEnumerablePropertyNamesSet.c_title_string,
								UnoDocumentStoringEnumerablePropertyNamesSet.c_encryptionData_NamedValuesSequence,
								UnoDocumentStoringEnumerablePropertyNamesSet.c_password_string,
								UnoDocumentStoringEnumerablePropertyNamesSet.c_editingPasswordInformation_any,
								UnoDocumentStoringEnumerablePropertyNamesSet.c_overwrites_boolean
							);
							List <Any> l_documentStoringPropertyValues = ListsFactory.createList <Any> (
								UnoDatumConverter.getAny (l_fileStoringFilterName),
								UnoDatumConverter.getAny (UnoPropertiesHandler.buildPropertiesArray (l_documentStoringFilterDataInPropertiesArrayPropertyNames, l_documentStoringFilterDataInPropertiesArrayPropertyValues)),
								UnoDatumConverter.getAny (l_documentStoringFilterDataInStringValue),
								UnoDatumConverter.getAny (a_fileConversionOrderExtendedJsonDatumParseEventsHandler.getTitle ()),
								UnoDatumConverter.getAny (l_encryptionDatumArray),
								UnoDatumConverter.getAny (l_encryptingPassword),
								l_editingPasswordInformation,
								UnoDatumConverter.getAny (l_overwrites)
							);
							l_documentStoringPropertiesArray = UnoPropertiesHandler.buildPropertiesArray (l_documentStoringPropertyNames, l_documentStoringPropertyValues);
						}
						else {
							l_documentStoringPropertiesArray = FilesConverter.createToCsvFileDocumentStoringPropertiesArray (a_fileConversionOrderExtendedJsonDatumParseEventsHandler.getCsvItemsDelimiterCharacterCode (), a_fileConversionOrderExtendedJsonDatumParseEventsHandler.getCsvTextItemQuotationCharacterCode (), a_fileConversionOrderExtendedJsonDatumParseEventsHandler.getCsvCharactersEncodingCode (), a_fileConversionOrderExtendedJsonDatumParseEventsHandler.getCsvAllTextItemsAreQuoted (), a_fileConversionOrderExtendedJsonDatumParseEventsHandler.getCsvContentsAreExportedAsShown (), a_fileConversionOrderExtendedJsonDatumParseEventsHandler.getCsvFormulaeThemselvesAreExported ());
						}
						if (!(a_fileConversionOrderExtendedJsonDatumParseEventsHandler.getAllSpreadSheetsAreExported ())) {
							l_conversionHasSucceeded = a_filesConverter.convertFile (l_originalFileUrl, a_fileConversionOrderExtendedJsonDatumParseEventsHandler.getOpeningPassword (), l_targetFileUrl, l_documentStoringPropertiesArray, l_unoDocumentTailors);
						}
						else {
							l_conversionHasSucceeded = a_filesConverter.convertSpreadSheetsDocumentFileToCsvFiles (l_originalFileUrl, a_fileConversionOrderExtendedJsonDatumParseEventsHandler.getOpeningPassword (), l_targetFileUrl, l_documentStoringPropertiesArray, l_unoDocumentTailors, a_fileConversionOrderExtendedJsonDatumParseEventsHandler.getTargetFileNamingRule (), a_fileConversionOrderExtendedJsonDatumParseEventsHandler.getHiddenSpreadSheetsAreExported ());
						}
					}
					return l_conversionHasSucceeded;
				}
			}
		}
	}
}

