package theBiasPlanet.filesConverter.programs;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.InputStreamReader;
import java.io.Reader;
import java.io.StringReader;
import java.nio.charset.StandardCharsets;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.text.NumberFormat;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.LocalTime;
import java.util.ArrayList;
import java.util.Locale;
import java.util.HashMap;
import com.sun.star.beans.NamedValue;
import com.sun.star.beans.PropertyValue;
import theBiasPlanet.coreUtilities.constantsGroups.GeneralConstantsConstantsGroup;
import theBiasPlanet.coreUtilities.collectionsHandling.ListsFactory;
import theBiasPlanet.coreUtilities.collectionsHandling.MapsFactory;
import theBiasPlanet.coreUtilities.cryptographyHandling.Hasher;
import theBiasPlanet.coreUtilities.inputsHandling.ReaderHandler;
import theBiasPlanet.coreUtilities.jsonDataHandling.ExtendedJsonDatumParseEventsHandler;
import theBiasPlanet.coreUtilities.jsonDataHandling.ExtendedJsonDatumParser;
import theBiasPlanet.coreUtilities.messagingHandling.Publisher;
import theBiasPlanet.coreUtilities.performanceMeasuringHandling.PerformanceMeasurer;
import theBiasPlanet.coreUtilities.stringsHandling.StringHandler;
import theBiasPlanet.coreUtilities.stringsHandling.StringTokenizer;
import theBiasPlanet.unoUtilities.connection.UnoConnection;
import theBiasPlanet.unoUtilities.connectionsHandling.UnoConnectionConnector;
import theBiasPlanet.unoUtilities.constantsGroups.UnoCharactersEncodingCodesConstantsGroup;
import theBiasPlanet.unoUtilities.constantsGroups.UnoDocumentStoringFilterNamesConstantsGroup;
import theBiasPlanet.unoUtilities.constantsGroups.UnoSizesConstantsGroup;
import theBiasPlanet.unoUtilities.constantsGroups.propertyNamesSets.*;
import theBiasPlanet.unoUtilities.cryptographyHandling.MicrosoftPasswordsHasher;
import theBiasPlanet.unoUtilities.displayElements.UnoDesktop;
import theBiasPlanet.unoUtilities.documentsHandling.UnoDocumentTailor;
import theBiasPlanet.unoUtilities.documentsHandling.UnoOpenXmlDocumentSetEditingPasswordTailor;
import theBiasPlanet.unoUtilities.documentsHandling.UnoSpreadSheetsDocumentMoveSpecifiedSheetTo1stPositionTailor;
import theBiasPlanet.unoUtilities.documentsHandling.UnoSpreadSheetsDocumentSetPageStylesTailor;
import theBiasPlanet.unoUtilities.documentsHandling.UnoTextDocumentSetPageStylesTailor;
import theBiasPlanet.unoUtilities.fileConversionsHandling.FilesConverter;
import theBiasPlanet.unoUtilities.nameValuePairsHandling.UnoNameValuePairsHandler;
import theBiasPlanet.unoUtilities.processesHandling.UnoProcessEnvironment;
import theBiasPlanet.unoUtilities.propertiesHandling.UnoPropertiesHandler;
import theBiasPlanet.unoUtilities.unoObjectsContexts.RemoteUnoObjectsContext;

public class FilesConverterConsoleProgram {
	private static final int c_conversionsOrderFilePathsStartIndexInArgumentsArray;
	public static class FileConversionOrderExtendedJsonDatumParseEventsHandler extends ExtendedJsonDatumParseEventsHandler {
		static private final String c_disabledKey;
		static private final String c_originalFileUrlKey;
		static private final String c_targetFileUrlKey;
		static private final String c_fileStoringFilterNameKey;
		static private final String c_openingPasswordKey;
		static private final String c_titleKey;
		static private final String c_encryptingPasswordKey;
		static private final String c_editingPasswordKey;
		static private final String c_certificateIssuerNameKey;
		static private final String c_certificatePasswordKey;
		static private final String c_signatureTimeStampAuthorityUrlKey;
		static private final String c_unoDocumentTailorNamesKey;
		static private final String c_allSpreadSheetsAreExportedKey;
		static private final String c_targetFileNamingRuleKey;
		static private final String c_csvItemsDelimiterCharacterCodeKey;
		static private final String c_csvTextItemQuotationCharacterCodeKey;
		static private final String c_csvCharactersEncodingCodeKey;
		static private final String c_csvAllTextItemsAreQuotedKey;
		static private final String c_csvContentsAreExportedAsShownKey;
		static private final String c_csvFormulaeThemselvesAreExportedKey;
		static private final String c_hiddenSpreadSheetsAreExportedKey;
		private String i_currentKey;
		private boolean i_disabled;
		private String i_originalFileUrl;
		private String i_targetFileUrl;
		private String i_fileStoringFilterName;
		private String i_openingPassword;
		private String i_title;
		private String i_encryptingPassword;
		private String i_editingPassword;
		private String i_certificateIssuerName;
		private String i_certificatePassword;
		private	String i_signatureTimeStampAuthorityUrl;
		private	ArrayList <String> i_unoDocumentTailorNames;
		private boolean i_allSpreadSheetsAreExported;
		private	int i_targetFileNamingRule;
		private int i_csvItemsDelimiterCharacterCode;
		private int i_csvTextItemQuotationCharacterCode;
		private int i_csvCharactersEncodingCode;
		private boolean i_csvAllTextItemsAreQuoted;
		private boolean i_csvContentsAreExportedAsShown;
		private boolean i_csvFormulaeThemselvesAreExported;
		private boolean i_hiddenSpreadSheetsAreExported;
		
		static {
			c_disabledKey = "Disabled";
			c_originalFileUrlKey = "OriginalFileUrl";
			c_targetFileUrlKey = "TargetFileUrl";
			c_fileStoringFilterNameKey = "FileStoringFilterName";
			c_openingPasswordKey = "OpeningPassword";
			c_titleKey = "Title";
			c_encryptingPasswordKey = "EncryptingPassword";
			c_editingPasswordKey = "EditingPassword";
			c_certificateIssuerNameKey = "CertificateIssuerName";
			c_certificatePasswordKey = "CertificatePassword";
			c_signatureTimeStampAuthorityUrlKey = "SignatureTimeStampAuthorityUrl";
			c_unoDocumentTailorNamesKey = "DocumentTailorNames";
			c_allSpreadSheetsAreExportedKey = "AllSpreadSheetsAreExported";
			c_targetFileNamingRuleKey = "TargetFileNamingRule";
			c_csvItemsDelimiterCharacterCodeKey = "CsvItemsDelimiterCharacterCode";
			c_csvTextItemQuotationCharacterCodeKey = "CsvTextItemQuotationCharacterCode";
			c_csvCharactersEncodingCodeKey = "CsvCharactersEncodingCode";
			c_csvAllTextItemsAreQuotedKey = "CsvAllTextItemsAreQuoted";
			c_csvContentsAreExportedAsShownKey = "CsvContentsAreExportedAsShown";
			c_csvFormulaeThemselvesAreExportedKey = "CsvFormulaeThemselvesAreExported";
			c_hiddenSpreadSheetsAreExportedKey = "HiddenSpreadSheetsAreExported";
		}
		
		@Override
		public void initialize () {
			i_currentKey = null;
			i_disabled = false;
			i_originalFileUrl = null;
			i_targetFileUrl = null;
			i_fileStoringFilterName = null;
			i_openingPassword = null;
			i_title = null;
			i_encryptingPassword = null;
			i_editingPassword = null;
			i_certificateIssuerName = null;
			i_certificatePassword = null;
			i_signatureTimeStampAuthorityUrl = null;
			i_unoDocumentTailorNames = null;
			i_allSpreadSheetsAreExported = false;
			i_targetFileNamingRule = 0;
			i_csvItemsDelimiterCharacterCode = Character.codePointAt (",", 0);
			i_csvTextItemQuotationCharacterCode = Character.codePointAt ("\"", 0);
			i_csvCharactersEncodingCode = UnoCharactersEncodingCodesConstantsGroup.c_utf8; // 76: UTF-8, 65535: UCS-2, 65534: UCS-4, 11: US-ASCII, 69: EUC_JP, 64: SHIFT_JIS
			i_csvAllTextItemsAreQuoted = false;
			i_csvContentsAreExportedAsShown = false;
			i_csvFormulaeThemselvesAreExported = false;
			i_hiddenSpreadSheetsAreExported = false;
		}
		
		public boolean getDisabled () {
			return i_disabled;
		}
		
		public String getOriginalFileUrl () {
			return i_originalFileUrl;
		}
		
		public String getTargetFileUrl () {
			return i_targetFileUrl;
		}
		
		public String getFileStoringFilterName () {
			return i_fileStoringFilterName;
		}
		
		public String getOpeningPassword () {
			return i_openingPassword;
		}
		
		public String getTitle () {
			return i_title;
		}
		
		public String getEncryptingPassword () {
			return i_encryptingPassword;
		}
		
		public String getEditingPassword () {
			return i_editingPassword;
		}
		
		public String getCertificateIssuerName () {
			return i_certificateIssuerName;
		}
		
		public String getCertificatePassword () {
			return i_certificatePassword;
		}
		
		public String getSignatureTimeStampAuthorityUrl () {
			return i_signatureTimeStampAuthorityUrl;
		}
		
		public ArrayList <String> getUnoDocumentTailorNames () {
			return i_unoDocumentTailorNames;
		}
		
		public boolean getAllSpreadSheetsAreExported () {
			return i_allSpreadSheetsAreExported;
		}
		
		public int getTargetFileNamingRule () {
			return i_targetFileNamingRule;
		}
		
		public int getCsvItemsDelimiterCharacterCode () {
			return i_csvItemsDelimiterCharacterCode;
		}
		
		public int getCsvTextItemQuotationCharacterCode () {
			return i_csvTextItemQuotationCharacterCode;
		}
		
		public int getCsvCharactersEncodingCode () {
			return i_csvCharactersEncodingCode;
		}
		
		public boolean getCsvAllTextItemsAreQuoted () {
			return i_csvAllTextItemsAreQuoted;
		}
		
		public boolean getCsvContentsAreExportedAsShown () {
			return i_csvContentsAreExportedAsShown;
		}
		
		public boolean getCsvFormulaeThemselvesAreExported () {
			return i_csvFormulaeThemselvesAreExported;
		}
		
		public boolean getHiddenSpreadSheetsAreExported () {
			return i_hiddenSpreadSheetsAreExported;
		}
		
		@Override
		public boolean onBooleanFound (Boolean a_value) throws Exception {
			boolean l_value = a_value.booleanValue ();
			if (i_currentKey.equals (c_disabledKey)) {
				 i_disabled = l_value;
				 if (i_disabled) {
					 return false;
				 }
			}
			else if (i_currentKey.equals (c_allSpreadSheetsAreExportedKey)) {
				 i_allSpreadSheetsAreExported = l_value;
			}
			else if (i_currentKey.equals (c_csvAllTextItemsAreQuotedKey)) {
				i_csvAllTextItemsAreQuoted = l_value;
			}
			else if (i_currentKey.equals (c_csvContentsAreExportedAsShownKey)) {
				i_csvContentsAreExportedAsShown = l_value;
			}
			else if (i_currentKey.equals (c_csvFormulaeThemselvesAreExportedKey)) {
				i_csvFormulaeThemselvesAreExported = l_value;
			}
			else if (i_currentKey.equals (c_hiddenSpreadSheetsAreExportedKey)) {
				i_hiddenSpreadSheetsAreExported = l_value;
			}
			i_currentKey = null;
			return true;
		}
		
		@Override
		public boolean onIntegerFound (Integer a_value) throws Exception {
			int l_value = a_value.intValue ();
			if (i_currentKey.equals (c_targetFileNamingRuleKey)) {
				i_targetFileNamingRule = l_value;
			}
			else if (i_currentKey.equals (c_csvItemsDelimiterCharacterCodeKey)) {
				i_csvItemsDelimiterCharacterCode = l_value;
			}
			else if (i_currentKey.equals (c_csvTextItemQuotationCharacterCodeKey)) {
				i_csvTextItemQuotationCharacterCode = l_value;
			}
			else if (i_currentKey.equals (c_csvCharactersEncodingCodeKey)) {
				i_csvCharactersEncodingCode = l_value;
			}
			i_currentKey = null;
			return true;
		}
		
		@Override
		public boolean onDoubleFound (Double a_value) throws Exception {
			return true;
		}
		
		@Override
		public boolean onLocalDateAndTimeFound (LocalDateTime a_value) throws Exception {
			return true;
		}
		
		@Override
		public boolean onLocalDateFound (LocalDate a_value) throws Exception {
			return true;
		}
		
		@Override
		public boolean onLocalTimeFound (LocalTime a_value) throws Exception {
			return true;
		}
		
		@Override
		public boolean onBytesArrayFound (Reader a_reader) throws Exception {
			return true;
		}
		
		@Override
		public boolean onStringFound (Reader a_reader) throws Exception {
			String l_string = ReaderHandler.getWholeString (a_reader);
			if (i_currentKey == null) {
				i_currentKey = l_string;
				if (i_currentKey.equals (c_unoDocumentTailorNamesKey)) {
					i_unoDocumentTailorNames = new ArrayList <String> ();
				}
			}
			else {
				if (i_currentKey.equals (c_originalFileUrlKey)) {
					i_originalFileUrl = StringHandler.effectuateEnvironmentVariables (l_string);
				}
				else if (i_currentKey.equals (c_targetFileUrlKey)) {
					i_targetFileUrl = StringHandler.effectuateEnvironmentVariables (l_string);
				}
				else if (i_currentKey.equals (c_fileStoringFilterNameKey)) {
					i_fileStoringFilterName = l_string;
				}
				else if (i_currentKey.equals (c_openingPasswordKey)) {
					i_openingPassword = l_string;
				}
				else if (i_currentKey.equals (c_titleKey)) {
					i_title = l_string;
				}
				else if (i_currentKey.equals (c_encryptingPasswordKey)) {
					i_encryptingPassword = l_string;
				}
				else if (i_currentKey.equals (c_editingPasswordKey)) {
					i_editingPassword = l_string;
				}
				else if (i_currentKey.equals (c_certificateIssuerNameKey)) {
					i_certificateIssuerName = l_string;
				}
				else if (i_currentKey.equals (c_certificatePasswordKey)) {
					i_certificatePassword = l_string;
				}
				else if (i_currentKey.equals (c_signatureTimeStampAuthorityUrlKey)) {
					i_signatureTimeStampAuthorityUrl = l_string;
				}
				else if (i_currentKey.equals (c_unoDocumentTailorNamesKey)) {
					i_unoDocumentTailorNames.add (l_string);
				}
				if (! (i_currentKey.equals (c_unoDocumentTailorNamesKey))) {
					i_currentKey = null;
				}
			}
			return true;
		}
		
		@Override
		public boolean onArrayStarted () throws Exception {
			return true;
		}
		
		@Override
		public boolean onArrayEnded () throws Exception {
			if (i_currentKey.equals (c_unoDocumentTailorNamesKey)) {
				i_currentKey = null;
			}
			return true;
		}
		
		@Override
		public boolean onDictionaryStarted () throws Exception {
			return true;
		}
		
		@Override
		public boolean onDictionaryEnded () throws Exception {
			return true;
		}
	}
	
	static {
		c_conversionsOrderFilePathsStartIndexInArgumentsArray = 1;
	}
	
	public static void main (String [] a_argumentsArray) throws Exception {
		int l_resultStatus = -1;
		try {
			if (a_argumentsArray.length < 2) {
				throw new Exception ("The arguments have to be these.\nThe argument 1: the server URL like 'socket,host=localhost,port=2002,tcpNoDelay=1;urp;StarOffice.ComponentContext'\nThe argument 2: the path of the conversion orders JSON file\n");
			}
			String l_unoServerUrl = a_argumentsArray [0];
			UnoProcessEnvironment l_localUnoProcessEnvironment = new UnoProcessEnvironment (LocalDateTime.now ().toString (), null);
			UnoConnectionConnector l_unoConnectionConnector = new UnoConnectionConnector (l_localUnoProcessEnvironment.getLocalObjectsContext ());
			UnoConnection l_unoConnection = l_unoConnectionConnector.connect (l_unoServerUrl, null);
			if (l_unoConnection == null) {
				throw new Exception (String.format ("%s", "The UNO connection could not be established."));
			}
			Publisher.logNormalInformation ("### A connection has been established.");
			RemoteUnoObjectsContext l_remoteUnoObjectsContext = l_unoConnection.getRemoteUnoObjectsContext ();
			FilesConverter l_filesConverter = new FilesConverter (UnoDesktop.getInstance (l_remoteUnoObjectsContext));
			HashMap <String, UnoDocumentTailor> l_unoDocumentTailorNameToUnoDocumentTailorMap = MapsFactory. <String, UnoDocumentTailor>createLinkedHashMap ("UnoSpreadSheetsDocumentMoveSpecifiedSheetTo1stPositionTailor_1", new UnoSpreadSheetsDocumentMoveSpecifiedSheetTo1stPositionTailor (l_remoteUnoObjectsContext, 1), "UnoTextDocumentSetPageStylesTailor_a4Landscape", new UnoTextDocumentSetPageStylesTailor (l_remoteUnoObjectsContext, MapsFactory. <String, Object>createLinkedHashMap (UnoTextDocumentPageStylePropertyNamesSet.c_pageSize_com_sun_star_awt_Size, UnoSizesConstantsGroup.c_a4Landscape, UnoTextDocumentPageStylePropertyNamesSet.c_pageIsLandscape_boolean, Boolean.valueOf (true))), "UnoSpreadSheetsDocumentSetPageStylesTailor_a4Portrait", new UnoSpreadSheetsDocumentSetPageStylesTailor (l_remoteUnoObjectsContext, MapsFactory. <String, Object>createLinkedHashMap (UnoSpreadSheetsDocumentPageStylePropertyNamesSet.c_pageSize_com_sun_star_awt_Size, UnoSizesConstantsGroup.c_a4Portrait, UnoSpreadSheetsDocumentPageStylePropertyNamesSet.c_pageIsLandscape_boolean, Boolean.valueOf (false))), "UnoOpenXmlDocumentSetEditingPasswordTailor_1", new UnoOpenXmlDocumentSetEditingPasswordTailor (l_remoteUnoObjectsContext, "PasswordB"));
			ArrayList <Thread> l_conversionsThreads = new ArrayList <Thread> ();
			PerformanceMeasurer.setStartTime ();
			for (int l_conversionOrdersFilePathIndex = GeneralConstantsConstantsGroup.c_iterationStartNumber; l_conversionOrdersFilePathIndex < a_argumentsArray.length - c_conversionsOrderFilePathsStartIndexInArgumentsArray; l_conversionOrdersFilePathIndex ++) {
				Path l_conversionOrdersFilePath = Paths.get (a_argumentsArray [c_conversionsOrderFilePathsStartIndexInArgumentsArray + l_conversionOrdersFilePathIndex]);
				Thread l_conversionsThread = new Thread ( () -> {
					try {
						executeConversionOrders (l_conversionOrdersFilePath, l_filesConverter, l_unoDocumentTailorNameToUnoDocumentTailorMap);
					}
					catch (Exception l_exception) {
						Publisher.logErrorInformation (l_exception);
					}
				});
				l_conversionsThreads.add (l_conversionsThread);
				l_conversionsThread.start ();
			}
			for (Thread l_conversionsThread: l_conversionsThreads) {
				l_conversionsThread.join ();
			}
			System.out.println (String.format ("### The elapsed time is %s ns.", NumberFormat.getNumberInstance (Locale.US).format (PerformanceMeasurer.getElapseTimeInNanoSeconds ())));
			l_unoConnection.disconnect ();
			l_resultStatus = 0;
		}
		catch (Exception l_exception) {
			Publisher.logErrorInformation (l_exception);
			System.exit (l_resultStatus);
		}
		System.exit (l_resultStatus);
	}
	
	public static void executeConversionOrders (Path a_conversionOrdersFilePath, FilesConverter a_filesConverter, HashMap <String, UnoDocumentTailor> a_unoDocumentTailorNameToUnoDocumentTailorMap) throws Exception {
		int l_conversionOrderIndex = GeneralConstantsConstantsGroup.c_iterationStartNumber;
		String l_conversionOrder = null;
		StringBuilder l_failedConversionOrderIndicesStringBuilder = new StringBuilder ();
		boolean l_conversionHasSucceeded = false;
		int l_jsonDictionaryLineStartIndex = GeneralConstantsConstantsGroup.c_unspecifiedInteger;
		int l_jsonDictionaryLineEndIndex = GeneralConstantsConstantsGroup.c_unspecifiedInteger;
		BufferedReader l_conversionOrdersReader = Files.newBufferedReader (a_conversionOrdersFilePath, StandardCharsets.UTF_8);
		ExtendedJsonDatumParser l_conversionOrderExtendedJsonDatumParser = new ExtendedJsonDatumParser ();
		FileConversionOrderExtendedJsonDatumParseEventsHandler l_fileConversionOrderExtendedJsonDatumParseEventsHandler = new FileConversionOrderExtendedJsonDatumParseEventsHandler ();
		while (true) {
			l_conversionHasSucceeded = false;
			l_conversionOrder = l_conversionOrdersReader.readLine ();
			if (l_conversionOrder == null) {
				break;
			}
			else {
				if (l_conversionOrder.startsWith (String.valueOf (GeneralConstantsConstantsGroup.c_jsonArrayOpener)) || l_conversionOrder.startsWith (String.valueOf (GeneralConstantsConstantsGroup.c_jsonArrayCloser))) {
					l_conversionOrderIndex ++;
					continue;
				}
				l_jsonDictionaryLineStartIndex = l_conversionOrder.indexOf (GeneralConstantsConstantsGroup.c_jsonDictionaryOpener);
				l_jsonDictionaryLineEndIndex = l_conversionOrder.lastIndexOf (GeneralConstantsConstantsGroup.c_jsonDictionaryCloser);
				if (l_jsonDictionaryLineStartIndex < GeneralConstantsConstantsGroup.c_iterationStartNumber || l_jsonDictionaryLineEndIndex < GeneralConstantsConstantsGroup.c_iterationStartNumber) {
					Publisher.logErrorInformation (String.format ("The conversion order of the index, '%d', doesn't contain any valid JSON dictionary.'\n", l_conversionOrderIndex));
				}
				else {
					l_conversionOrder = l_conversionOrder.substring (l_jsonDictionaryLineStartIndex, l_jsonDictionaryLineEndIndex + 1);
					try {
						l_conversionHasSucceeded = executeConversionOrder (l_conversionOrder, a_filesConverter, a_unoDocumentTailorNameToUnoDocumentTailorMap, l_conversionOrderExtendedJsonDatumParser, l_fileConversionOrderExtendedJsonDatumParseEventsHandler);
					}
					catch (Exception l_exception) {
						Publisher.logErrorInformation (String.format ("The order of the index, '%d' has failed: %s.", l_conversionOrderIndex, l_exception.toString ()));
					}
					if (!l_conversionHasSucceeded) {
						if (l_failedConversionOrderIndicesStringBuilder.length () == 0) {
						}
						else {
							l_failedConversionOrderIndicesStringBuilder.append (",");
						}
						l_failedConversionOrderIndicesStringBuilder.append (String.format ("%d", l_conversionOrderIndex));
					}
				}
			}
			l_conversionOrderIndex ++;
		}
		l_conversionOrdersReader.close ();
		System.out.println (String.format ("### The failed conversion order indices of '%s' are {%s}.", a_conversionOrdersFilePath.toString (), l_failedConversionOrderIndicesStringBuilder.toString ()));
	}
	
	public static boolean executeConversionOrder (String a_conversionOrder, FilesConverter a_filesConverter, HashMap <String, UnoDocumentTailor> a_unoDocumentTailorNameToUnoDocumentTailorMap, ExtendedJsonDatumParser a_conversionOrderExtendedJsonDatumParser, FileConversionOrderExtendedJsonDatumParseEventsHandler a_fileConversionOrderExtendedJsonDatumParseEventsHandler) throws Exception {
		boolean l_conversionHasSucceeded = false;
		Boolean l_overwrites = Boolean.valueOf (true);
		String l_originalFileUrl = null;
		String l_targetFileUrl = null;
		String l_fileStoringFilterName = null;
		String l_encryptingPassword = null;
		ArrayList <String> l_encryptionDatumNames = ListsFactory. <String>createArrayList ("OOXPassword", "CryptoType");
		ArrayList <Object> l_encryptionDatumValues = null;
		NamedValue [] l_encryptionDatumArray = null;
		String l_editingPassword = null;
		ArrayList <String> l_editingPasswordInformationPropertyNames = ListsFactory. <String>createArrayList ("algorithm-name", "salt", "iteration-count", "hash");
		int l_saltLength = 16;
		byte [] l_salt = Hasher.createSalt (l_saltLength);
		int l_numberOfIterationForHashing = 100000;
		int l_editingPasswordHashLength = 16;
		byte [] l_editingPasswordHash = null;
		ArrayList <Object> l_editingPasswordInformationPropertyValues = null;
		PropertyValue [] l_editingPasswordInformationPropertiesArray = null;
		int l_editingPasswordMicrosoftHash = GeneralConstantsConstantsGroup.c_unspecifiedInteger;
		Object l_editingPasswordInformation = null;
		String l_certificateIssuerName = null;
		String l_signatureTimeStampAuthorityUrl = null;
		ArrayList <String> l_documentStoringFilterDataInPropertiesArrayPropertyNames = null;
		ArrayList <Object> l_documentStoringFilterDataInPropertiesArrayPropertyValues = null;
		String l_documentStoringFilterDataInStringValue = null;
		PropertyValue [] l_documentStoringPropertiesArray = null;
		ArrayList <UnoDocumentTailor> l_unoDocumentTailors = null;
		a_conversionOrderExtendedJsonDatumParser.parse (new StringReader (a_conversionOrder), a_fileConversionOrderExtendedJsonDatumParseEventsHandler);
		l_originalFileUrl = a_fileConversionOrderExtendedJsonDatumParseEventsHandler.getOriginalFileUrl ();
		l_targetFileUrl = a_fileConversionOrderExtendedJsonDatumParseEventsHandler.getTargetFileUrl ();
		l_fileStoringFilterName = a_fileConversionOrderExtendedJsonDatumParseEventsHandler.getFileStoringFilterName ();
		if (a_fileConversionOrderExtendedJsonDatumParseEventsHandler.getDisabled ()) {
			return true;
		}
		if (l_originalFileUrl == null) {
			throw new Exception ("The original file URL specification is missing.");
		}
		else if (l_targetFileUrl == null) {
			throw new Exception ("The target file URL specification is missing.");
		}
		else if (l_fileStoringFilterName == null) {
			throw new Exception ("The filter name specification is missing.");
		}
		else {
			ArrayList <String> l_unoDocumentTailorNames = a_fileConversionOrderExtendedJsonDatumParseEventsHandler.getUnoDocumentTailorNames ();
			if (l_unoDocumentTailorNames != null) {
				l_unoDocumentTailors = new ArrayList <UnoDocumentTailor> ();
				for (String l_unoDocumentTailorName: l_unoDocumentTailorNames) {
					l_unoDocumentTailors.add (a_unoDocumentTailorNameToUnoDocumentTailorMap.get (l_unoDocumentTailorName));
				}
			}
			if (!(l_fileStoringFilterName.equals (UnoDocumentStoringFilterNamesConstantsGroup.c_toCsvFileFilterName))) {
				l_encryptingPassword = a_fileConversionOrderExtendedJsonDatumParseEventsHandler.getEncryptingPassword ();
				if (l_encryptingPassword != null) {
					if (l_fileStoringFilterName.equals (UnoDocumentStoringFilterNamesConstantsGroup.c_toMicrosoftWord2007XmlFileFilterName) || l_fileStoringFilterName.equals (UnoDocumentStoringFilterNamesConstantsGroup.c_toMicrosoftExcel2007XmlFileFilterName) || l_fileStoringFilterName.equals (UnoDocumentStoringFilterNamesConstantsGroup.c_toMicrosoftPowerPoint2007XmlFileFilterName)) {
						l_encryptionDatumValues = ListsFactory. <Object>createArrayList (l_encryptingPassword, "Standard");
						l_encryptionDatumArray = UnoNameValuePairsHandler.buildNameValuePairsArray (l_encryptionDatumNames, l_encryptionDatumValues);
						l_encryptingPassword = null;
					}
					else if (l_fileStoringFilterName.equals (UnoDocumentStoringFilterNamesConstantsGroup.c_toPdfFileFilterNameForTextDocuments) || l_fileStoringFilterName.equals (UnoDocumentStoringFilterNamesConstantsGroup.c_toPdfFileFilterNameForSpreadSheetsDocuments) || l_fileStoringFilterName.equals (UnoDocumentStoringFilterNamesConstantsGroup.c_toPdfFileFilterNameForPresentationDocuments)) {
						if (l_documentStoringFilterDataInPropertiesArrayPropertyNames == null) {
							l_documentStoringFilterDataInPropertiesArrayPropertyNames = ListsFactory. <String>createArrayList ();
							l_documentStoringFilterDataInPropertiesArrayPropertyValues = ListsFactory. <Object>createArrayList ();
						}
						l_documentStoringFilterDataInPropertiesArrayPropertyNames.add (UnoPdfExportingPropertyNamesSet.c_fileIsEncrypted_boolean);
						l_documentStoringFilterDataInPropertiesArrayPropertyNames.add (UnoPdfExportingPropertyNamesSet.c_openingPassword_string);
						l_documentStoringFilterDataInPropertiesArrayPropertyValues.add (Boolean.valueOf (true));
						l_documentStoringFilterDataInPropertiesArrayPropertyValues.add (l_encryptingPassword);
						l_encryptingPassword = null;
					}
				}
				l_editingPassword = a_fileConversionOrderExtendedJsonDatumParseEventsHandler.getEditingPassword ();
				if (l_editingPassword != null) {
					if (!l_fileStoringFilterName.equals (UnoDocumentStoringFilterNamesConstantsGroup.c_toMicrosoftWord2007XmlFileFilterName) && !l_fileStoringFilterName.equals (UnoDocumentStoringFilterNamesConstantsGroup.c_toMicrosoftWord97FileFilterName) && !l_fileStoringFilterName.equals (UnoDocumentStoringFilterNamesConstantsGroup.c_toMicrosoftExcel2007XmlFileFilterName) && !l_fileStoringFilterName.equals (UnoDocumentStoringFilterNamesConstantsGroup.c_toMicrosoftExcel97FileFilterName) && !l_fileStoringFilterName.equals (UnoDocumentStoringFilterNamesConstantsGroup.c_toMicrosoftPowerPoint2007XmlFileFilterName) && !l_fileStoringFilterName.equals (UnoDocumentStoringFilterNamesConstantsGroup.c_toMicrosoftPowerPoint97FileFilterName)) {
						l_editingPasswordHash = Hasher.hashInPbkdf2 (l_editingPassword, l_salt, l_numberOfIterationForHashing, l_editingPasswordHashLength);
						l_editingPasswordInformationPropertyValues = ListsFactory. <Object>createArrayList ("PBKDF2", l_salt, l_numberOfIterationForHashing, l_editingPasswordHash);
						l_editingPasswordInformationPropertiesArray = UnoPropertiesHandler.buildPropertiesArray (l_editingPasswordInformationPropertyNames, l_editingPasswordInformationPropertyValues);
						l_editingPasswordInformation = l_editingPasswordInformationPropertiesArray;
					}
					else {
						if (l_fileStoringFilterName.equals (UnoDocumentStoringFilterNamesConstantsGroup.c_toMicrosoftWord2007XmlFileFilterName) || l_fileStoringFilterName.equals (UnoDocumentStoringFilterNamesConstantsGroup.c_toMicrosoftWord97FileFilterName) || l_fileStoringFilterName.equals (UnoDocumentStoringFilterNamesConstantsGroup.c_toMicrosoftExcel2007XmlFileFilterName) || l_fileStoringFilterName.equals (UnoDocumentStoringFilterNamesConstantsGroup.c_toMicrosoftPowerPoint2007XmlFileFilterName)) {
							l_editingPasswordMicrosoftHash = MicrosoftPasswordsHasher.hashIn32bits (l_editingPassword);
						}
						else {
							l_editingPasswordMicrosoftHash = MicrosoftPasswordsHasher.hashIn16bits (l_editingPassword);
						}
						l_editingPasswordInformation = Integer.valueOf (l_editingPasswordMicrosoftHash);
					}
				}
				l_certificateIssuerName = a_fileConversionOrderExtendedJsonDatumParseEventsHandler.getCertificateIssuerName ();
				if (l_certificateIssuerName != null) {
					if (l_fileStoringFilterName.equals (UnoDocumentStoringFilterNamesConstantsGroup.c_toPdfFileFilterNameForTextDocuments) || l_fileStoringFilterName.equals (UnoDocumentStoringFilterNamesConstantsGroup.c_toPdfFileFilterNameForSpreadSheetsDocuments) || l_fileStoringFilterName.equals (UnoDocumentStoringFilterNamesConstantsGroup.c_toPdfFileFilterNameForPresentationDocuments)) {
						if (l_documentStoringFilterDataInPropertiesArrayPropertyNames == null) {
							l_documentStoringFilterDataInPropertiesArrayPropertyNames = ListsFactory. <String>createArrayList ();
							l_documentStoringFilterDataInPropertiesArrayPropertyValues = ListsFactory. <Object>createArrayList ();
						}
						l_documentStoringFilterDataInPropertiesArrayPropertyNames.add (UnoPdfExportingPropertyNamesSet.c_fileIsSigned_boolean);
						l_documentStoringFilterDataInPropertiesArrayPropertyNames.add (UnoPdfExportingPropertyNamesSet.c_signatureCertificate_com_sun_star_security_XCertificate);
						l_documentStoringFilterDataInPropertiesArrayPropertyNames.add (UnoPdfExportingPropertyNamesSet.c_signatureCertificatePassword_string);
						l_documentStoringFilterDataInPropertiesArrayPropertyValues.add (Boolean.valueOf (true));
						l_documentStoringFilterDataInPropertiesArrayPropertyValues.add (a_filesConverter.getRemoteUnoObjectsContext ().getNetworkSecurityServicesCertificate (l_certificateIssuerName, null).getAddress ());
						l_documentStoringFilterDataInPropertiesArrayPropertyValues.add (a_fileConversionOrderExtendedJsonDatumParseEventsHandler.getCertificatePassword ());
					}
				}
				l_signatureTimeStampAuthorityUrl = a_fileConversionOrderExtendedJsonDatumParseEventsHandler.getSignatureTimeStampAuthorityUrl ();
				if (l_signatureTimeStampAuthorityUrl != null) {
					if (l_fileStoringFilterName.equals (UnoDocumentStoringFilterNamesConstantsGroup.c_toPdfFileFilterNameForTextDocuments) || l_fileStoringFilterName.equals (UnoDocumentStoringFilterNamesConstantsGroup.c_toPdfFileFilterNameForSpreadSheetsDocuments) || l_fileStoringFilterName.equals (UnoDocumentStoringFilterNamesConstantsGroup.c_toPdfFileFilterNameForPresentationDocuments)) {
						if (l_documentStoringFilterDataInPropertiesArrayPropertyNames == null) {
							l_documentStoringFilterDataInPropertiesArrayPropertyNames = ListsFactory. <String>createArrayList ();
							l_documentStoringFilterDataInPropertiesArrayPropertyValues = ListsFactory. <Object>createArrayList ();
						}
						l_documentStoringFilterDataInPropertiesArrayPropertyNames.add (UnoPdfExportingPropertyNamesSet.c_signatureTimeStampAuthorityUrl_string);
						l_documentStoringFilterDataInPropertiesArrayPropertyValues.add (l_signatureTimeStampAuthorityUrl);
					}
				}
				ArrayList <String> l_documentStoringPropertyNames = ListsFactory. <String>createArrayList (
					UnoDocumentStoringEnumerablePropertyNamesSet.c_filterName_string,
					UnoDocumentStoringEnumerablePropertyNamesSet.c_filterData_any,
					UnoDocumentStoringEnumerablePropertyNamesSet.c_filterData_string,
					UnoDocumentStoringEnumerablePropertyNamesSet.c_title_string,
					UnoDocumentStoringEnumerablePropertyNamesSet.c_encryptionData_NamedValuesSequence,
					UnoDocumentStoringEnumerablePropertyNamesSet.c_password_string,
					UnoDocumentStoringEnumerablePropertyNamesSet.c_editingPasswordInformation_any,
					UnoDocumentStoringEnumerablePropertyNamesSet.c_overwrites_boolean
				);
				ArrayList <Object> l_documentStoringPropertyValues = ListsFactory. <Object>createArrayList (
					l_fileStoringFilterName,
					UnoPropertiesHandler.buildPropertiesArray (l_documentStoringFilterDataInPropertiesArrayPropertyNames, l_documentStoringFilterDataInPropertiesArrayPropertyValues),
					l_documentStoringFilterDataInStringValue,
					a_fileConversionOrderExtendedJsonDatumParseEventsHandler.getTitle (),
					l_encryptionDatumArray,
					l_encryptingPassword,
					l_editingPasswordInformation,
					l_overwrites
				);
				l_documentStoringPropertiesArray = UnoPropertiesHandler.buildPropertiesArray (l_documentStoringPropertyNames, l_documentStoringPropertyValues);
			}
			else {
				l_documentStoringPropertiesArray = FilesConverter.createToCsvFileDocumentStoringPropertiesArray (a_fileConversionOrderExtendedJsonDatumParseEventsHandler.getCsvItemsDelimiterCharacterCode (), a_fileConversionOrderExtendedJsonDatumParseEventsHandler.getCsvTextItemQuotationCharacterCode (), a_fileConversionOrderExtendedJsonDatumParseEventsHandler.getCsvCharactersEncodingCode (), a_fileConversionOrderExtendedJsonDatumParseEventsHandler.getCsvAllTextItemsAreQuoted (), a_fileConversionOrderExtendedJsonDatumParseEventsHandler.getCsvContentsAreExportedAsShown (), a_fileConversionOrderExtendedJsonDatumParseEventsHandler.getCsvFormulaeThemselvesAreExported ());
			}
			if (!(a_fileConversionOrderExtendedJsonDatumParseEventsHandler.getAllSpreadSheetsAreExported ())) {
				l_conversionHasSucceeded = a_filesConverter.convertFile (l_originalFileUrl, a_fileConversionOrderExtendedJsonDatumParseEventsHandler.getOpeningPassword (), l_targetFileUrl, l_documentStoringPropertiesArray, l_unoDocumentTailors);
			}
			else {
				l_conversionHasSucceeded = a_filesConverter.convertSpreadSheetsDocumentFileToCsvFiles (l_originalFileUrl, a_fileConversionOrderExtendedJsonDatumParseEventsHandler.getOpeningPassword (), l_targetFileUrl, l_documentStoringPropertiesArray, l_unoDocumentTailors, a_fileConversionOrderExtendedJsonDatumParseEventsHandler.getTargetFileNamingRule (), a_fileConversionOrderExtendedJsonDatumParseEventsHandler.getHiddenSpreadSheetsAreExported ());
			}
		}
		return l_conversionHasSucceeded;
	}
}

