#include "theBiasPlanet/unoUtilities/connection/UnoConnection.hpp"
#include <com/sun/star/container/ElementExistException.hpp>
#include <com/sun/star/frame/TerminationVetoException.hpp>
#include <com/sun/star/frame/XDesktop.hpp>
#include <com/sun/star/uno/Any.h>
#include "theBiasPlanet/coreUtilities/messagingHandling/Publisher.hpp"
#include "theBiasPlanet/unoUtilities/constantsGroups/UnoDefaultValuesConstantsGroup.hpp"
#include "theBiasPlanet/unoUtilities/constantsGroups/propertyNamesSets/UnoObjectsContextPropertyNamesSet.hpp"
#include "theBiasPlanet/unoUtilities/displayElements/UnoDesktop.hpp"
#include "theBiasPlanet/unoUtilities/officeInstance/OfficeInstance.hpp"
#include "theBiasPlanet/unoUtilities/stringsHandling/UnoExtendedStringHandler.hpp"
#include "theBiasPlanet/unoUtilities/unoDataHandling/UnoDatumConverter.hpp"

using namespace ::com::sun::star::uno;
using namespace ::theBiasPlanet::coreUtilities::messagingHandling;
using namespace ::theBiasPlanet::unoUtilities::constantsGroups;
using namespace ::theBiasPlanet::unoUtilities::constantsGroups::propertyNamesSets;
using namespace ::theBiasPlanet::unoUtilities::displayElements;
using namespace ::theBiasPlanet::unoUtilities::officeInstance;
using namespace ::theBiasPlanet::unoUtilities::stringsHandling;
using namespace ::theBiasPlanet::unoUtilities::unoDataHandling;

namespace theBiasPlanet {
	namespace unoUtilities {
		namespace connection {
			UnoConnection::InitialUnoObjectsProvider::InitialUnoObjectsProvider (UnoObjectPointer <LocalUnoObjectsContext> & a_localUnoObjectsContext): ImplInheritanceHelper1 <UnoComponentBase, XInstanceProvider> (string ("::theBiasPlanet::unoUtilities::connection::UnoConnection::InitialUnoObjectsProvider")), i_localUnoObjectsContext (a_localUnoObjectsContext) {
			}
			
			UnoConnection::InitialUnoObjectsProvider::~InitialUnoObjectsProvider () {
			}
			
			Reference <XInterface> SAL_CALL UnoConnection::InitialUnoObjectsProvider::getInstance (OUString const & a_initialUnoObjectName) {
				if (UnoDefaultValuesConstantsGroup::c_initiallyOfferedUnoObjectName == UnoExtendedStringHandler::getString (a_initialUnoObjectName)) {
					// The template type cannot be 'XInterface' because it is ambiguous in 'UnoObjectsContext'; the 'Reference <XComponentContext>' is automatically converted to a return type instance.
					return i_localUnoObjectsContext.getPointerInReference <XComponentContext> ();
				}
				else {
					return Reference <XInterface> ();
				}
			}
			
			UnoConnection::OfficeTerminationQueryListener::OfficeTerminationQueryListener (UnoObjectPointer <UnoConnection> const & a_unoConnection): ImplInheritanceHelper1 <UnoComponentBase, XTerminateListener> (string ("::theBiasPlanet::unoUtilities::connection::UnoConnection::OfficeTerminationQueryListener")), i_unoConnection (a_unoConnection) {
			}
			
			UnoConnection::OfficeTerminationQueryListener::~OfficeTerminationQueryListener () {
			}
			
			void SAL_CALL UnoConnection::OfficeTerminationQueryListener::queryTermination (::com::sun::star::lang::EventObject const & a_event) {
				Publisher::logNormalInformation (string ("### The termination of the office instance is queried."));
				UnoObjectPointer <RemoteUnoObjectsContext> & l_remoteUnoObjectsContext (i_unoConnection->getRemoteUnoObjectsContext ());
				if (l_remoteUnoObjectsContext->officeInstanceSingletonIsSet ()) {
					if (! (l_remoteUnoObjectsContext->serverIsPlannedToBeTerminated ()) || l_remoteUnoObjectsContext->getCriticalSessionCount () > 0) {
						Publisher::logNormalInformation (string ("### And vetoed."));
						throw TerminationVetoException ();
					}
					else {
						Publisher::logNormalInformation (string ("### And accepted."));
					}
				}
			}
			
			void SAL_CALL UnoConnection::OfficeTerminationQueryListener::notifyTermination (::com::sun::star::lang::EventObject const & a_event) {
				Publisher::logNormalInformation (string ("### The termination of the office instance is notified."));
				//i_unoConnection->disconnect ();
			}
			
			void SAL_CALL UnoConnection::OfficeTerminationQueryListener::disposing (::com::sun::star::lang::EventObject const & a_event) {
			}
			
			UnoConnection::UnoConnection (UnoObjectPointer <XComponentContext> const & a_originalRemoteUnoObjectsContext, string const & a_unoConnectionIdentification, UnoObjectPointer <XBridge> & a_unoBridge, UnoObjectPointer <InitialUnoObjectsProvider> const & a_initialUnoObjectsProvider, optional <list <UnoConnectionEventsListener *>> const & a_eventsListeners): ImplInheritanceHelper1 <UnoComponentBase, ::com::sun::star::lang::XEventListener> (string ("::theBiasPlanet::unoUtilities::connection::UnoConnection")), i_remoteUnoObjectsContext (new RemoteUnoObjectsContext (a_originalRemoteUnoObjectsContext, a_unoConnectionIdentification)), i_unoBridge (a_unoBridge), i_initialUnoObjectsProvider (a_initialUnoObjectsProvider), i_eventsListeners (a_eventsListeners) {
				i_remoteUnoObjectsContext->setUpForOfficeInstance (i_remoteUnoObjectsContext);
				i_unoBridge->addEventListener (Reference <XEventListener> (this));
				if (i_eventsListeners.has_value ()) {
					list <UnoConnectionEventsListener *>::iterator l_eventsListenersIterator;
					for (l_eventsListenersIterator = i_eventsListeners->begin (); l_eventsListenersIterator != i_eventsListeners->end (); l_eventsListenersIterator ++) {
						(*l_eventsListenersIterator)->connected (this);
					}
				}
				// add local singletons Start
				// * if any singleton is added, it has to be removed in the destructor of this class.
				i_remoteUnoObjectsContext->addLocalProperty (UnoObjectsContextPropertyNamesSet::c_unoConnectionSingleton_string, new UnoObjectPointer <UnoConnection> (this));
				// add local singletons End
				UnoObjectPointer <UnoDesktop> * l_unoDesktop = (UnoObjectPointer <UnoDesktop> *) i_remoteUnoObjectsContext->getLocalPropertyValue (UnoObjectsContextPropertyNamesSet::c_unoDesktopSingleton_string);
				if (l_unoDesktop != nullptr) {
					i_officeTerminationQueryListener = UnoObjectPointer <OfficeTerminationQueryListener> (new OfficeTerminationQueryListener (this));
					(*l_unoDesktop)->getUnderlyingUnoObject ().getPointerInReference <XDesktop> ()->addTerminateListener ( (*i_officeTerminationQueryListener).getPointerInReference <XTerminateListener> ());
				}
				else {
					i_officeTerminationQueryListener = nullopt;
				}
			}
			
			UnoConnection::~UnoConnection () {
			}
			
			UnoObjectPointer <RemoteUnoObjectsContext> & UnoConnection::getRemoteUnoObjectsContext () {
				return i_remoteUnoObjectsContext;
			}
			
			void UnoConnection::disconnect () {
				if (!i_unoBridge.isEmpty ()) {
					try {
						if (i_officeTerminationQueryListener.has_value ()) {
							UnoObjectPointer <UnoDesktop> * l_unoDesktop = (UnoObjectPointer <UnoDesktop> *) i_remoteUnoObjectsContext->getLocalPropertyValue (UnoObjectsContextPropertyNamesSet::c_unoDesktopSingleton_string);
							(*l_unoDesktop)->getUnderlyingUnoObject ().getPointerInReference <XDesktop> ()->removeTerminateListener (i_officeTerminationQueryListener->getPointerInReference <XTerminateListener> ());
						}
						i_unoBridge->dispose ();
					}
					catch (Exception & l_exception) {
						Publisher::logErrorInformation (StringHandler::format (string ("### UnoConnection::disconnect error: %s."), UnoExtendedStringHandler::getString (l_exception.Message)));
					}
					catch (exception & l_exception) {
						Publisher::logErrorInformation (StringHandler::format (string ("### UnoConnection::disconnect error: %s."), string (l_exception.what ())));
					}
				}
			}
			
			void UnoConnection::disposing (EventObject const & a_event) {
				if (i_eventsListeners.has_value ()) {
					list <UnoConnectionEventsListener *>::iterator l_eventsListenersIterator;
					for (l_eventsListenersIterator = i_eventsListeners->begin (); l_eventsListenersIterator != i_eventsListeners->end (); l_eventsListenersIterator ++) {
						(*l_eventsListenersIterator)->disconnected (this);
					}
				}
				i_remoteUnoObjectsContext.clear ();
				i_unoBridge.clear ();
				i_initialUnoObjectsProvider.clear ();
				i_eventsListeners = nullopt;
				i_officeTerminationQueryListener = nullopt;
			}
		}
	}
}

