#include "theBiasPlanet/unoUtilities/inspectionsHandling/UnoComponentInspector.hpp"
#include <stdexcept>
#include <com/sun/star/beans/XPropertySet.hpp>
#include <com/sun/star/beans/XPropertySetInfo.hpp>
#include <com/sun/star/lang/XTypeProvider.hpp>
#include <com/sun/star/uno/Sequence.hxx>
#include <com/sun/star/uno/Type.hxx>
#include <cppu/unotype.hxx>
#include "theBiasPlanet/coreUtilities/constantsGroups/GeneralConstantsConstantsGroup.hpp"
#include "theBiasPlanet/coreUtilities/stringsHandling/StringHandler.hpp"
#include "theBiasPlanet/unoUtilities/inspectionsHandling/UnoInterfaceNotImplementedException.hpp"
#include "theBiasPlanet/unoUtilities/stringsHandling/UnoExtendedStringHandler.hpp"
#include "theBiasPlanet/unoUtilities/unoDataHandling/UnoDatumConverter.hpp"

using namespace ::com::sun::star::beans;
using namespace ::com::sun::star::lang;
using namespace ::com::sun::star::uno;
using namespace ::theBiasPlanet::coreUtilities::constantsGroups;
using namespace ::theBiasPlanet::coreUtilities::stringsHandling;
using namespace ::theBiasPlanet::unoUtilities::stringsHandling;
using namespace ::theBiasPlanet::unoUtilities::unoDataHandling;

namespace theBiasPlanet {
	namespace unoUtilities {
		namespace inspectionsHandling {
			list <string> UnoComponentInspector::getUnoTypeNames (void * const a_unoObjectInXInterface) {
				Reference <XTypeProvider> l_unoObjectInXTypeProvider (Reference <XInterface> ( (XInterface *) a_unoObjectInXInterface), UNO_QUERY);
				if (l_unoObjectInXTypeProvider.is ()) {
					Sequence <Type> l_UnoTypes = l_unoObjectInXTypeProvider->getTypes ();
					int l_numberOfUnoTypes = l_UnoTypes.getLength ();
					list <string> l_unoTypeNames;
					for (int l_unoTypeIndex = 0; l_unoTypeIndex < l_numberOfUnoTypes; l_unoTypeIndex ++) {
						l_unoTypeNames.push_back (UnoExtendedStringHandler::getString (l_UnoTypes [l_unoTypeIndex].getTypeName ()));
					}
					return l_unoTypeNames;
				}
				else {
					throw UnoInterfaceNotImplementedException ("XTypeProvider");
				}
			}
			
			NavigableLinkedMap <string, Any> UnoComponentInspector::getPropertyNameToValueMap (void * const a_unoObjectInXInterface) {
				Reference <XPropertySet> l_unoObjectInXPropertySet (Reference <XInterface> ( (XInterface *) a_unoObjectInXInterface), UNO_QUERY);
				if (l_unoObjectInXPropertySet.is ()) {
					NavigableLinkedMap <string, Any> l_propertyNameToValueMap;
					// do not use 'UnoObjectPointer' for storing 'l_propertySetInfoInXPropertySetInfo', because an eternal loop would happen.
					Reference <XPropertySetInfo> l_propertySetInfoInXPropertySetInfo (l_unoObjectInXPropertySet->getPropertySetInfo ());
					if (l_propertySetInfoInXPropertySetInfo.is ()) {
						Sequence <Property> l_properties (l_propertySetInfoInXPropertySetInfo->getProperties ());
						int l_numberOfProperties = l_properties.getLength ();
						for (int l_propertyIndex = GeneralConstantsConstantsGroup::c_iterationStartNumber; l_propertyIndex < l_numberOfProperties; l_propertyIndex ++) {
							try {
								l_propertyNameToValueMap [UnoExtendedStringHandler::getString (l_properties [l_propertyIndex].Name)] = l_unoObjectInXPropertySet->getPropertyValue (l_properties [l_propertyIndex].Name);
							}
							catch (UnknownPropertyException const & l_exception) {
								l_propertyNameToValueMap [UnoExtendedStringHandler::getString (l_properties [l_propertyIndex].Name)] = UnoDatumConverter::convertToUnoAnyDatum (string ("Not Found"));
							}
							catch (exception const & l_exception) {
								l_propertyNameToValueMap [UnoExtendedStringHandler::getString (l_properties [l_propertyIndex].Name)] = UnoDatumConverter::convertToUnoAnyDatum (StringHandler::format <string> (string (": ### An error has occurred in getting the property value: exception is %s."), l_exception.what ()));
							}
						}
						return l_propertyNameToValueMap;
					}
					else {
						return l_propertyNameToValueMap;
					}
				}
				else {
					throw UnoInterfaceNotImplementedException ("XPropertySet");
				}
			}
		}
	}
}

