#include "theBiasPlanet/unoUtilities/officeInstance/OfficeInstance.hpp"
#include <thread>
#include <com/sun/star/container/ElementExistException.hpp>
#include <com/sun/star/container/NoSuchElementException.hpp>
#include <com/sun/star/frame/XDesktop.hpp>
#include <com/sun/star/lang/XComponent.hpp>
#include <com/sun/star/util/CloseVetoException.hpp>
#include <com/sun/star/util/XCloseable.hpp>
#include "theBiasPlanet/coreUtilities/constantsGroups/GeneralConstantsConstantsGroup.hpp"
#include "theBiasPlanet/coreUtilities/messagingHandling/Publisher.hpp"
#include "theBiasPlanet/coreUtilities/stringsHandling/StringHandler.hpp"
#include "theBiasPlanet/unoUtilities/constantsGroups/propertyNamesSets/UnoObjectsContextPropertyNamesSet.hpp"
#include "theBiasPlanet/unoUtilities/documents/UnoDocument.hpp"
#include "theBiasPlanet/unoUtilities/stringsHandling/UnoExtendedStringHandler.hpp"

using namespace ::com::sun::star::container;
using namespace ::com::sun::star::frame;
using namespace ::com::sun::star::lang;
using namespace ::com::sun::star::uno;
using namespace ::com::sun::star::util;
using namespace ::theBiasPlanet::coreUtilities::constantsGroups;
using namespace ::theBiasPlanet::coreUtilities::messagingHandling;
using namespace ::theBiasPlanet::coreUtilities::stringsHandling;
using namespace ::theBiasPlanet::unoUtilities::constantsGroups;
using namespace ::theBiasPlanet::unoUtilities::constantsGroups::propertyNamesSets;
using namespace ::theBiasPlanet::unoUtilities::documents;
using namespace ::theBiasPlanet::unoUtilities::stringsHandling;

namespace theBiasPlanet {
	namespace unoUtilities {
		namespace officeInstance {
			OfficeInstance::OfficeInstance (UnoObjectPointer <RemoteUnoObjectsContext> & a_remoteUnoObjectsContext): ImplInheritanceHelper1 <UnoComponentBase, XOfficeInstance> (string ("::theBiasPlanet::unoUtilities::officeInstance::OfficeInstance")), i_remoteUnoObjectsContext (a_remoteUnoObjectsContext), i_criticalSessionCount (0), i_serverIsPlannedToBeTerminated (false), i_unoDesktop ( * ( (UnoObjectPointer <UnoDesktop> *) i_remoteUnoObjectsContext->getLocalPropertyValue (UnoObjectsContextPropertyNamesSet::c_unoDesktopSingleton_string))) {
				if (! (i_remoteUnoObjectsContext->getValueByName (UnoExtendedStringHandler::getOustring (UnoObjectsContextPropertyNamesSet::c_unoOfficeInstanceSingleton_string)).hasValue ())) {
					try {
						i_remoteUnoObjectsContext->addExtraGlobalProperty (UnoObjectsContextPropertyNamesSet::c_unoOfficeInstanceSingleton_string, Any (Reference <XOfficeInstance> (this)));
					}
					catch (ElementExistException l_exception) {
					}
				}
			}
			
			OfficeInstance::~OfficeInstance  () {
			}
			
			sal_Bool SAL_CALL OfficeInstance::incrementCriticalSessionCount (OUString const & a_connectionIdentification) {
				unique_lock <recursive_mutex> l_lock (i_threadsMutex);
				if (i_serverIsPlannedToBeTerminated) {
					return sal_False;
				}
				else {
					i_criticalSessionCount ++;
					Publisher::logDebugInformation (StringHandler::format (string ("### the critical sessions count has been incremented to '%d'."), i_criticalSessionCount));
					return sal_True;
				}
			}
			
			void SAL_CALL OfficeInstance::decrementCriticalSessionCount (OUString const & a_connectionIdentification) {
				unique_lock <recursive_mutex> l_lock (i_threadsMutex);
				i_criticalSessionCount --;
				Publisher::logDebugInformation (StringHandler::format (string ("### the critical sessions count has been decremented to '%d'."), i_criticalSessionCount));
				if (i_criticalSessionCount <= 0) {
					i_threadsCondition.notify_all ();
				}
			}
			
			sal_Int32 SAL_CALL OfficeInstance::getCriticalSessionCount () {
				unique_lock <recursive_mutex> l_lock (i_threadsMutex);
				return i_criticalSessionCount;
			}
			
			sal_Bool SAL_CALL OfficeInstance::serverIsPlannedToBeTerminated () {
				unique_lock <recursive_mutex> l_lock (i_threadsMutex);
				return i_serverIsPlannedToBeTerminated;
			}
			
			void OfficeInstance::setServerAsPlannedToBeTerminated () {
				unique_lock <recursive_mutex> l_lock (i_threadsMutex);
				i_serverIsPlannedToBeTerminated = true;
			}
			
			bool OfficeInstance::waitUntilNoCriticalSession (int a_timeOutInMilliseconds) {
				unique_lock <recursive_mutex> l_lock (i_threadsMutex);
				if (i_criticalSessionCount > 0) {
					if (a_timeOutInMilliseconds == GeneralConstantsConstantsGroup::c_unspecifiedInteger) {
						i_threadsCondition.wait (l_lock);
						return true;
					}
					else {
						if (i_threadsCondition.wait_for (l_lock, chrono::milliseconds (a_timeOutInMilliseconds)) == cv_status::no_timeout) {
							return true;
						}
						else {
							return false;
						}
					}
				}
				else {
					return true;
				}
			}
			
			bool OfficeInstance::terminate (int a_gracePeriodInMillisecondsBeforeForcing) {
				setServerAsPlannedToBeTerminated ();
				waitUntilNoCriticalSession (a_gracePeriodInMillisecondsBeforeForcing);
				list <UnoObjectPointer <XComponent>> l_unoObjects (i_unoDesktop->getChildObjects ());
				for (UnoObjectPointer <XComponent> l_unoObject: l_unoObjects) {
					if (! (l_unoObject.getPointerInReference <XCloseable> ().is ())) {
						continue;
					}
					try {
						UnoDocument l_unoDocument (i_remoteUnoObjectsContext, UnoObjectPointer <XModel> (l_unoObject));
						Publisher::logWarningInformation (StringHandler::format (string ("The document '%s' is left opened."), l_unoDocument.getUrl ()));
						try {
							l_unoDocument.close ();
							Publisher::logWarningInformation (StringHandler::format (string ("The document '%s' has been closed."), l_unoDocument.getUrl ()));
						}
						catch (CloseVetoException const & l_exception) {
							Publisher::logWarningInformation (StringHandler::format (string ("The document '%s' has been vetoed to be closed."), l_unoDocument.getUrl ()));
							l_unoObject->dispose ();
							Publisher::logWarningInformation (StringHandler::format (string ("The document '%s' has been disposed."), l_unoDocument.getUrl ()));
						}
						catch (DisposedException const & l_exception) {
							Publisher::logWarningInformation (StringHandler::format (string ("The document '%s' has been disposed."), l_unoDocument.getUrl ()));
						}
					}
					catch (exception const & l_exception) {
						Publisher::logErrorInformation (StringHandler::format (string ("An error has occurred: %s."), string (l_exception.what ())));
					}
				}
				try {
					i_remoteUnoObjectsContext->removeExtraGlobalProperty (UnoObjectsContextPropertyNamesSet::c_unoOfficeInstanceSingleton_string);
				}
				catch (NoSuchElementException l_exception) {
				}
				bool l_shutdownTryResult (false);
				// loops because the shutdown may be vetoed.
				while (true) {
					try {
						l_shutdownTryResult = i_unoDesktop->getUnderlyingUnoObject ().getPointerInReference <XDesktop> ()->terminate ();
					}
					catch (DisposedException l_exception) {
						return true;
					}
					if (l_shutdownTryResult) {
						return l_shutdownTryResult;
					}
					this_thread::sleep_for (milliseconds (1000));
				}
			}
		}
	}
}

