#include "theBiasPlanet/unoUtilities/unoDataHandling/UnoDatumConverter.hpp"

#include <iomanip>
#include <iostream>
#include <regex>
#include <com/sun/star/uno/Any.hxx>
#include <com/sun/star/uno/Type.hxx>
#include <cppu/unotype.hxx>
#include "theBiasPlanet/unoUtilities/collectionsHandling/UnoSequenceHandler.hpp"
#include "theBiasPlanet/unoUtilities/constantsGroups/UnoRegularExpressionsConstantsGroup.hpp"
#include "theBiasPlanet/unoUtilities/stringsHandling/UnoExtendedStringHandler.hpp"

using namespace ::theBiasPlanet::unoUtilities::collectionsHandling;
using namespace ::theBiasPlanet::unoUtilities::constantsGroups
;
using namespace ::theBiasPlanet::unoUtilities::stringsHandling;

namespace theBiasPlanet {
	namespace unoUtilities {
		namespace unoDataHandling  {
			bool UnoDatumConverter::convertToNonUnoDatum (sal_Bool const & a_unoDatum) {
				return (bool) a_unoDatum;
			}
			
			unsigned char UnoDatumConverter::convertToNonUnoDatum (sal_Int8 const & a_unoDatum) {
				return (unsigned char) a_unoDatum;
			}
			
			short UnoDatumConverter::convertToNonUnoDatum (sal_Int16 const & a_unoDatum) {
				return (short) a_unoDatum;
			}
			
			long UnoDatumConverter::convertToNonUnoDatum (sal_Int32 const & a_unoDatum) {
				return (long) a_unoDatum;
			}
			
			long long UnoDatumConverter::convertToNonUnoDatum (sal_Int64 const & a_unoDatum) {
				return (long long) a_unoDatum;
			}
			
			float UnoDatumConverter::convertToNonUnoDatum (float const & a_unoDatum) {
				return a_unoDatum;
			}
			
			double UnoDatumConverter::convertToNonUnoDatum (double const & a_unoDatum) {
				return a_unoDatum;
			}
			
			char UnoDatumConverter::convertToNonUnoDatum (sal_Unicode const & a_unoDatum) {
				return (char) a_unoDatum;
			}
			
			string UnoDatumConverter::convertToNonUnoDatum (OUString const & a_unoDatum) {
				return UnoExtendedStringHandler::getString (a_unoDatum);
			}
			
			string UnoDatumConverter::convertToNonUnoDatum (Reference <XInterface> const & a_unoDatum) {
				return string ("A UNO object");
			}
			
			string UnoDatumConverter::convertToNonUnoDatum (Type const & a_unoDatum) {
				return UnoExtendedStringHandler::getString (a_unoDatum.getTypeName ());
			}
			
			string UnoDatumConverter::convertToNonUnoDatum (Any const & a_unoDatum) {
				ostringstream l_stringStream;
				TypeClass l_containedDatumType = a_unoDatum.getValueTypeClass ();
				if (l_containedDatumType == TypeClass_SEQUENCE) {
					string l_containdeDatumTypeName = UnoExtendedStringHandler::getString (a_unoDatum.getValueTypeName ());
					l_stringStream << l_containdeDatumTypeName << ":";
					if (l_containdeDatumTypeName == "[]boolean") {
						Sequence <sal_Bool> * l_unoSequence = (Sequence <sal_Bool> *) a_unoDatum.getValue ();
						l_stringStream << UnoSequenceHandler::toString (*l_unoSequence);
					}
					else if (l_containdeDatumTypeName == "[]byte") {
						Sequence <sal_Int8> * l_unoSequence = (Sequence <sal_Int8> *) a_unoDatum.getValue ();
						l_stringStream << UnoSequenceHandler::toString (*l_unoSequence);
					}
					else if (l_containdeDatumTypeName == "[]short") {
						Sequence <sal_Int16> * l_unoSequence = (Sequence <sal_Int16> *) a_unoDatum.getValue ();
						l_stringStream << UnoSequenceHandler::toString (*l_unoSequence);
					}
					else if (l_containdeDatumTypeName == "[]long") {
						Sequence <sal_Int32> * l_unoSequence = (Sequence <sal_Int32> *) a_unoDatum.getValue ();
						l_stringStream << UnoSequenceHandler::toString (*l_unoSequence);
					}
					else if (l_containdeDatumTypeName == "[]hyper") {
						Sequence <sal_Int64> * l_unoSequence = (Sequence <sal_Int64> *) a_unoDatum.getValue ();
						l_stringStream << UnoSequenceHandler::toString (*l_unoSequence);
					}
					else if (l_containdeDatumTypeName == "[]float") {
						Sequence <float> * l_unoSequence = (Sequence <float> *) a_unoDatum.getValue ();
						l_stringStream << UnoSequenceHandler::toString (*l_unoSequence);
					}
					else if (l_containdeDatumTypeName == "[]double") {
						Sequence <double> * l_unoSequence = (Sequence <double> *) a_unoDatum.getValue ();
						l_stringStream << UnoSequenceHandler::toString (*l_unoSequence);
					}
					else if (l_containdeDatumTypeName == "[]char") {
						Sequence <sal_Unicode> * l_unoSequence = (Sequence <sal_Unicode> *) a_unoDatum.getValue ();
						l_stringStream << UnoSequenceHandler::toString (*l_unoSequence);
					}
					else if (l_containdeDatumTypeName == "[]string") {
						Sequence <OUString> * l_unoSequence = (Sequence <OUString> *) a_unoDatum.getValue ();
						l_stringStream << UnoSequenceHandler::toString (*l_unoSequence);
					}
					else if (l_containdeDatumTypeName == "[]interface") {
						Sequence <Reference <XInterface>> * l_unoSequence = (Sequence <Reference <XInterface>> *) a_unoDatum.getValue ();
						l_stringStream << UnoSequenceHandler::toString (*l_unoSequence);
					}
					else if (l_containdeDatumTypeName == "[]type") {
						Sequence <Type> * l_unoSequence = (Sequence <Type> *) a_unoDatum.getValue ();
						l_stringStream << UnoSequenceHandler::toString (*l_unoSequence);
					}
					else if (l_containdeDatumTypeName == "[]any") {
						Sequence <Any> * l_unoSequence = (Sequence <Any> *) a_unoDatum.getValue ();
						l_stringStream << UnoSequenceHandler::toString (*l_unoSequence);
					}
					else {
						// TODO
						l_stringStream << "An unsupported sequence";
					}
				}
				else if (l_containedDatumType == TypeClass_BOOLEAN) {
					l_stringStream << *((sal_Bool *) a_unoDatum.getValue ());
				}
				else if (l_containedDatumType == TypeClass_BYTE) {
					l_stringStream << *((sal_Int8 *) a_unoDatum.getValue ());
				}
				else if (l_containedDatumType == TypeClass_SHORT) {
					l_stringStream << *((sal_Int16 *) a_unoDatum.getValue ());
				}
				else if (l_containedDatumType == TypeClass_LONG) {
					l_stringStream << *((sal_Int32 *) a_unoDatum.getValue ());
				}
				else if (l_containedDatumType == TypeClass_HYPER) {
					l_stringStream << *((sal_Int64 *) a_unoDatum.getValue ());
				}
				else if (l_containedDatumType == TypeClass_FLOAT) {
					l_stringStream << *((float *) a_unoDatum.getValue ());
				}
				else if (l_containedDatumType == TypeClass_DOUBLE) {
					l_stringStream << *((double *) a_unoDatum.getValue ());
				}
				else if (l_containedDatumType == TypeClass_CHAR) {
					l_stringStream << *((sal_Unicode *) a_unoDatum.getValue ());
				}
				else if (l_containedDatumType == TypeClass_STRING) {
					l_stringStream << UnoExtendedStringHandler::getString (*((OUString *) a_unoDatum.getValue ()));
				}
				else if (l_containedDatumType == TypeClass_INTERFACE) {
					//l_stringStream << UnoExtendedStringHandler::getString (*((Reference *) a_unoDatum.getValue ()));
					l_stringStream << "A UNO object";
				}
				else if (l_containedDatumType == TypeClass_TYPE) {
					l_stringStream << UnoExtendedStringHandler::getString (((Type *) a_unoDatum.getValue ())->getTypeName ());
				}
				return l_stringStream.str ();
			}
			
			bool UnoDatumConverter::convertToBool (Any const & a_unoDatum) {
				return convertToNonUnoDatum (*((sal_Bool *) a_unoDatum.getValue ()));
			}
			
			unsigned char UnoDatumConverter::convertToUnsignedChar (Any const & a_unoDatum) {
				return convertToNonUnoDatum (*((sal_Int8 *) a_unoDatum.getValue ()));
			}
			
			short UnoDatumConverter::convertToShort (Any const & a_unoDatum) {
					return convertToNonUnoDatum (*((sal_Int16 *) a_unoDatum.getValue ()));
			}
			
			long UnoDatumConverter::convertToLong (Any const & a_unoDatum) {
				return convertToNonUnoDatum (*((sal_Int32 *) a_unoDatum.getValue ()));
			}
			
			long long UnoDatumConverter::convertToLongLong (Any const & a_unoDatum) {
				return convertToNonUnoDatum (*((sal_Int64 *) a_unoDatum.getValue ()));
			}
			
			float UnoDatumConverter::convertToFloat (Any const & a_unoDatum) {
				return convertToNonUnoDatum (*((float *) a_unoDatum.getValue ()));
			}
			
			double UnoDatumConverter::convertToDouble (Any const & a_unoDatum) {
				return convertToNonUnoDatum (*((double *) a_unoDatum.getValue ()));
			}
			
			char UnoDatumConverter::convertToChar (Any const & a_unoDatum) {
				return convertToNonUnoDatum (*((sal_Unicode *) a_unoDatum.getValue ()));
			}
			
			string UnoDatumConverter::convertToString (Any const & a_unoDatum) {
				return convertToNonUnoDatum (*((OUString *) a_unoDatum.getValue ()));
			}
			
			string UnoDatumConverter::convertToTypeString (Any const & a_unoDatum) {
				return convertToNonUnoDatum (*((Type *) a_unoDatum.getValue ()));
			}
			
			list <bool> UnoDatumConverter::convertToBoolList (Any const & a_unoDatum) {
				return convertToNonUnoDatum <sal_Bool, bool> (*((Sequence <sal_Bool> *) a_unoDatum.getValue ()));
			}
			
			list <unsigned char> UnoDatumConverter::convertToUnsignedCharList (Any const & a_unoDatum) {
				return convertToNonUnoDatum <sal_Int8, unsigned char> (*((Sequence <sal_Int8> *) a_unoDatum.getValue ()));
			}
			
			list <short> UnoDatumConverter::convertToShortList (Any const & a_unoDatum) {
				return convertToNonUnoDatum <sal_Int16, short> (*((Sequence <sal_Int16> *) a_unoDatum.getValue ()));
			}
			
			list <long> UnoDatumConverter::convertToLongList (Any const & a_unoDatum) {
				return convertToNonUnoDatum <sal_Int32, long> (*((Sequence <sal_Int32> *) a_unoDatum.getValue ()));
			}
			
			list <long long> UnoDatumConverter::convertToLongLongList (Any const & a_unoDatum) {
				return convertToNonUnoDatum <sal_Int64, long long> (*((Sequence <sal_Int64> *) a_unoDatum.getValue ()));
			}
			
			list <float> UnoDatumConverter::convertToFloatList (Any const & a_unoDatum) {
				return convertToNonUnoDatum <float, float> (*((Sequence <float> *) a_unoDatum.getValue ()));
			}
			
			list <double> UnoDatumConverter::convertToDoubleList (Any const & a_unoDatum) {
				return convertToNonUnoDatum <double, double> (*((Sequence <double> *) a_unoDatum.getValue ()));
			}
			
			list <char> UnoDatumConverter::convertToCharList (Any const & a_unoDatum) {
				return convertToNonUnoDatum <sal_Unicode, char> (*((Sequence <sal_Unicode> *) a_unoDatum.getValue ()));
			}
			
			list <string> UnoDatumConverter::convertToStringList (Any const & a_unoDatum) {
				return convertToNonUnoDatum <OUString, string> (*((Sequence <OUString> *) a_unoDatum.getValue ()));
			}
			
			list <string> UnoDatumConverter::convertToTypeStringList (Any const & a_unoDatum) {
				return convertToNonUnoDatum <Type, string> (*((Sequence <Type> *) a_unoDatum.getValue ()));
			}
			
			sal_Bool UnoDatumConverter::convertToUnoDatum (bool const & a_nonUnoDatum) {
				return (sal_Bool) a_nonUnoDatum;
			}
			
			sal_Int8 UnoDatumConverter::convertToUnoDatum (unsigned char const & a_nonUnoDatum) {
				return (sal_Int8) a_nonUnoDatum;
			}
			
			sal_Int16 UnoDatumConverter::convertToUnoDatum (short const & a_nonUnoDatum) {
				return (sal_Int16) a_nonUnoDatum;
			}
			
			sal_Int32 UnoDatumConverter::convertToUnoDatum (long const & a_nonUnoDatum) {
				return (sal_Int32) a_nonUnoDatum;
			}
			
			sal_Int64 UnoDatumConverter::convertToUnoDatum (long long const & a_nonUnoDatum) {
				return (sal_Int64) a_nonUnoDatum;
			}
			
			float UnoDatumConverter::convertToUnoDatum (float const & a_nonUnoDatum) {
				return a_nonUnoDatum;
			}
			
			double UnoDatumConverter::convertToUnoDatum (double const & a_nonUnoDatum) {
				return a_nonUnoDatum;
			}
			
			sal_Unicode UnoDatumConverter::convertToUnoDatum (char const & a_nonUnoDatum) {
				return (sal_Unicode) a_nonUnoDatum;
			}
			
			OUString UnoDatumConverter::convertToUnoDatum (string const & a_nonUnoDatum) {
				return UnoExtendedStringHandler::getOustring (a_nonUnoDatum);
			}
			
			::com::sun::star::util::URL UnoDatumConverter::getUrlInURL (string const & a_url) {
				OUString const l_urlInOUString (UnoExtendedStringHandler::getOustring (a_url));
				OUString const l_emptyInOUString;
				int l_portNumber = 0;
				string::const_iterator l_urlIteratorAtStart (a_url.cbegin ());
				string::const_iterator l_urlIteratorAtEnd (a_url.cend ());
				smatch l_regularExpressionMatcher;
				if (regex_search (l_urlIteratorAtStart, l_urlIteratorAtEnd, l_regularExpressionMatcher, UnoRegularExpressionsConstantsGroup::c_urlRegularExpression)) {
					OUString const l_protocolInOUString (UnoExtendedStringHandler::getOustring (l_regularExpressionMatcher [1]));
					OUString const l_pathInOUString (UnoExtendedStringHandler::getOustring (l_regularExpressionMatcher [2]));
					return ::com::sun::star::util::URL (l_urlInOUString, l_urlInOUString, l_protocolInOUString, l_emptyInOUString, l_emptyInOUString, l_emptyInOUString, l_portNumber, l_pathInOUString, l_emptyInOUString, l_emptyInOUString, l_emptyInOUString);
				}
				return ::com::sun::star::util::URL (l_urlInOUString, l_urlInOUString, l_emptyInOUString, l_emptyInOUString, l_emptyInOUString, l_emptyInOUString, l_portNumber, l_emptyInOUString, l_emptyInOUString, l_emptyInOUString, l_emptyInOUString);
			}
		}
	}
}

