#include "theBiasPlanet/unoUtilities/unoObjectsContext/RemoteUnoObjectsContext.hpp"
#include <com/sun/star/uno/Exception.hpp>
#include <com/sun/star/xml/crypto/XSEInitializer.hpp>
#include <com/sun/star/xml/crypto/XXMLSecurityContext.hpp>
#include "theBiasPlanet/coreUtilities/constantsGroups/GeneralConstantsConstantsGroup.hpp"
#include "theBiasPlanet/unoUtilities/constantsGroups/UnoServiceNamesConstantsGroup.hpp"
#include "theBiasPlanet/unoUtilities/constantsGroups/propertyNamesSets/UnoObjectsContextPropertyNamesSet.hpp"
#include "theBiasPlanet/unoUtilities/displayElements/UnoDesktop.hpp"
#include "theBiasPlanet/unoUtilities/stringsHandling/UnoExtendedStringHandler.hpp"

using namespace ::theBiasPlanet::coreUtilities::constantsGroups;
using namespace ::theBiasPlanet::unoUtilities::constantsGroups::propertyNamesSets;
using namespace ::theBiasPlanet::unoUtilities::displayElements;
using namespace ::theBiasPlanet::unoUtilities::stringsHandling;

namespace theBiasPlanet {
	namespace unoUtilities {
		namespace unoObjectsContext {
			RemoteUnoObjectsContext::RemoteUnoObjectsContext (UnoObjectPointer <XComponentContext> const & a_underlyingUnoObject, string const & a_unoConnectionIdentification): UnoObjectsContext (a_underlyingUnoObject, a_unoConnectionIdentification) {
			}
			
			RemoteUnoObjectsContext::~RemoteUnoObjectsContext () {
				UnoObjectPointer <UnoDesktop> * l_unoDesktop = (UnoObjectPointer <UnoDesktop> *) getLocalPropertyValue (UnoObjectsContextPropertyNamesSet::c_unoDesktopSingleton_string);
				if (l_unoDesktop != nullptr) {
					delete l_unoDesktop;
					removeLocalProperty (UnoObjectsContextPropertyNamesSet::c_unoDesktopSingleton_string);
				}
			}
			
			bool RemoteUnoObjectsContext::setUpForOfficeInstance (UnoObjectPointer <RemoteUnoObjectsContext> & a_this) {
				try {
					UnoObjectPointer <UnoDesktop> l_unoDesktop (UnoDesktop::getInstance (a_this));
					if (! (l_unoDesktop.isEmpty ())) {
						addLocalProperty (UnoObjectsContextPropertyNamesSet::c_unoDesktopSingleton_string, new UnoObjectPointer <UnoDesktop> (l_unoDesktop));
						if (i_officeInstance.isEmpty ()) {
							Any l_officeInstance (getValueByName (UnoExtendedStringHandler::getOustring (UnoObjectsContextPropertyNamesSet::c_unoOfficeInstanceSingleton_string)));
							if (l_officeInstance.hasValue ()) {
								i_officeInstance.setPrimaryAddress <XInterface> (* ( (Reference <XInterface> const *) (l_officeInstance.getValue ())));
							}
						}
						return true;
					}
				}
				catch (Exception l_exception) {
					// the remote UNO environment may not be any Office instance, and it is OK.
				}
				return false;
			}
			
			bool RemoteUnoObjectsContext::officeInstanceSingletonIsSet () {
				return ! (i_officeInstance.isEmpty ());
			}
			
			bool RemoteUnoObjectsContext::incrementCriticalSessionCount (string const & a_connectionIdentification) {
				if (! (i_officeInstance.isEmpty ())) {
					return i_officeInstance->incrementCriticalSessionCount (UnoExtendedStringHandler::getOustring (a_connectionIdentification));
				}
				else {
					return false;
				}
			}
			
			void RemoteUnoObjectsContext::decrementCriticalSessionCount (string const & a_connectionIdentification) {
				if (! (i_officeInstance.isEmpty ())) {
					i_officeInstance->decrementCriticalSessionCount (UnoExtendedStringHandler::getOustring (a_connectionIdentification));
				}
			}
			
			int RemoteUnoObjectsContext::getCriticalSessionCount () {
				if (! (i_officeInstance.isEmpty ())) {
					return i_officeInstance->getCriticalSessionCount ();
				}
				else {
					return GeneralConstantsConstantsGroup::c_unspecifiedInteger;
				}
			}
			
			bool RemoteUnoObjectsContext::serverIsPlannedToBeTerminated () {
				if (! (i_officeInstance.isEmpty ())) {
					return i_officeInstance->serverIsPlannedToBeTerminated ();
				}
				else {
					return false;
				}
			}
			
			UnoObjectPointer <XCertificate> RemoteUnoObjectsContext::selectCertificateFromCertificatesSequence (Sequence <Reference <XCertificate>> const & a_certificatesSequenceInXCertificate, string const & a_certificateIssuerName, optional <Sequence <sal_Int8>> const & a_certificateSerialNumber) {
				for (Reference <XCertificate> l_certificateInXCertificate: a_certificatesSequenceInXCertificate) {
					if (a_certificateIssuerName == UnoExtendedStringHandler::getString (l_certificateInXCertificate->getIssuerName ())) {
						if (! (a_certificateSerialNumber.has_value ())) {
							return UnoObjectPointer <XCertificate> (l_certificateInXCertificate);
						}
						else {
							Sequence <sal_Int8> l_certificateSerialNumber = l_certificateInXCertificate->getSerialNumber ();
							if ((*a_certificateSerialNumber).getLength () == l_certificateSerialNumber.getLength ()) {
								int l_certificateSerialNumberByteIndex = GeneralConstantsConstantsGroup::c_iterationStartNumber;
								for (sal_Int8 l_certificateSerialNumberByte: l_certificateSerialNumber) {
									if ( (*a_certificateSerialNumber) [l_certificateSerialNumberByteIndex] != l_certificateSerialNumberByte) {
										break;
									}
									l_certificateSerialNumberByteIndex ++;
								}
								if (l_certificateSerialNumberByteIndex == l_certificateSerialNumber.getLength ()) {
									return UnoObjectPointer <XCertificate> (l_certificateInXCertificate);
								}
							}
						}
					}
				}
				return UnoObjectPointer <XCertificate> (nullptr);
			}
			
			UnoObjectPointer <XCertificate> RemoteUnoObjectsContext::getGnupgCertificate (string const & a_certificateIssuerName) {
				if (i_gnupgUnoSecurityEnvironment.isEmpty ()) {
					UnoObjectPointer <XInterface> l_gnupgSecurityEnvironmentInitializer (getUnoServiceInstance (UnoServiceNamesConstantsGroup::c_com_sun_star_xml_crypto_GPGSEInitializer, nullopt));
					UnoObjectPointer <XXMLSecurityContext> l_gnupgSecurityContext (l_gnupgSecurityEnvironmentInitializer.getPointerInReference <XSEInitializer> (true)->createSecurityContext (UnoExtendedStringHandler::getOustring (string ("Gnupg")))); 
					if (l_gnupgSecurityContext->getSecurityEnvironmentNumber () > 0) {
						i_gnupgUnoSecurityEnvironment.setPrimaryAddress (l_gnupgSecurityContext->getSecurityEnvironmentByIndex (GeneralConstantsConstantsGroup::c_iterationStartNumber));
					}
				}
				if (!i_gnupgUnoSecurityEnvironment.isEmpty ()) {
					Sequence <Reference <XCertificate>> l_gnupgCertificatesSequenceInXCertificate (i_gnupgUnoSecurityEnvironment->getAllCertificates ());
					return selectCertificateFromCertificatesSequence (l_gnupgCertificatesSequenceInXCertificate, a_certificateIssuerName, nullopt);
				}
				return UnoObjectPointer <XCertificate> (nullptr);
			}
			
			UnoObjectPointer <XCertificate> RemoteUnoObjectsContext::getNetworkSecurityServicesCertificate (string const & a_certificateIssuerName, optional <Sequence <sal_Int8>> const & a_certificateSerialNumber) {
				if (i_networkSecurityServicesUnoSecurityEnvironment.isEmpty ()) {
					UnoObjectPointer <XInterface> l_networkSecurityServicesSecurityEnvironmentInitializer (getUnoServiceInstance (UnoServiceNamesConstantsGroup::c_com_sun_star_xml_crypto_SEInitializer, nullopt));
					UnoObjectPointer <XXMLSecurityContext> l_networkSecurityServicesSecurityContext (l_networkSecurityServicesSecurityEnvironmentInitializer.getPointerInReference <XSEInitializer> (true)->createSecurityContext (UnoExtendedStringHandler::getOustring (string ("NetworkSecurityServices")))); 
					if (l_networkSecurityServicesSecurityContext->getSecurityEnvironmentNumber () > 0) {
						i_networkSecurityServicesUnoSecurityEnvironment.setPrimaryAddress (l_networkSecurityServicesSecurityContext->getSecurityEnvironmentByIndex (GeneralConstantsConstantsGroup::c_iterationStartNumber));
					}
					l_networkSecurityServicesSecurityEnvironmentInitializer.getPointerInReference <XSEInitializer> (true)->freeSecurityContext (l_networkSecurityServicesSecurityContext.getPointerInReference ());
				}
				if (!i_networkSecurityServicesUnoSecurityEnvironment.isEmpty ()) {
					Sequence <Reference <XCertificate>> l_networkSecurityServicesCertificatesSequenceInXCertificate (i_networkSecurityServicesUnoSecurityEnvironment->getPersonalCertificates ());
					return selectCertificateFromCertificatesSequence (l_networkSecurityServicesCertificatesSequenceInXCertificate, a_certificateIssuerName, a_certificateSerialNumber);
				}
				return UnoObjectPointer <XCertificate> ();
			}
		}
	}
}

