namespace theBiasPlanet {
	namespace unoUtilities {
		namespace connection {
			using System;
			using System.Collections.Generic;
			using uno;
			using unoidl.com.sun.star.bridge;
			using unoidl.com.sun.star.frame;
			using unoidl.com.sun.star.lang;
			using unoidl.com.sun.star.uno;
			using theBiasPlanet.coreUtilities.messagingHandling;
			using theBiasPlanet.unoUtilities.connectionsHandling;
			using theBiasPlanet.unoUtilities.constantsGroups;
			using theBiasPlanet.unoUtilities.constantsGroups.propertyNamesSets;
			using theBiasPlanet.unoUtilities.displayElements;
			using theBiasPlanet.unoUtilities.pointers;
			using theBiasPlanet.unoUtilities.unoComponentBases;
			using theBiasPlanet.unoUtilities.unoDataHandling;
			using theBiasPlanet.unoUtilities.unoObjectsContexts;
			
			public class UnoConnection: UnoComponentBase, XEventListener {
				public class InitialUnoObjectsProvider: UnoComponentBase, XInstanceProvider {
					private LocalUnoObjectsContext i_localUnoObjectsContext = null;
					
					public InitialUnoObjectsProvider (LocalUnoObjectsContext a_localUnoObjectsContext) {
						i_localUnoObjectsContext = a_localUnoObjectsContext;
					}
					
					~InitialUnoObjectsProvider () {
					}
					
					// 'XInterface' cannot be used for C#
					virtual public Object getInstance (String a_initialUnoObjectName) {
						if (UnoDefaultValuesConstantsGroup.c_initiallyOfferedUnoObjectName == a_initialUnoObjectName) {
							return i_localUnoObjectsContext;
						}
						else {
							return null;
						}
					}
				}
				public interface UnoConnectionEventsListener {
					void connected (UnoConnection a_unoConnection);
	
					void disconnected (UnoConnection a_unoConnection);
				}
				public class OfficeTerminationQueryListener: UnoComponentBase, XTerminateListener {
					private UnoConnection i_unoConnection;
					
					public OfficeTerminationQueryListener (UnoConnection a_unoConnection) {
						i_unoConnection = a_unoConnection;
					}
					
					~OfficeTerminationQueryListener () {
					}
					
					public void queryTermination (unoidl.com.sun.star.lang.EventObject a_event) {
						Publisher.logNormalInformation ("### The termination of the office instance is queried.");
						/*
						if (i_vetoesOfficeTermination) {
							throw new TerminationVetoException ();
						}
						*/
					}
					
					public void notifyTermination (unoidl.com.sun.star.lang.EventObject a_event) {
						Publisher.logNormalInformation ("### The termination of the office instance is notified.");
						//i_unoConnection.disconnect ();
					}
					
					public void disposing (unoidl.com.sun.star.lang.EventObject a_event) {
					}
				}
				private RemoteUnoObjectsContext i_remoteUnoObjectsContext;
				private XComponent i_unoBridge;
				private	InitialUnoObjectsProvider i_initialUnoObjectsProvider;
				private List <UnoConnectionEventsListener> i_eventsListeners;
				private OfficeTerminationQueryListener i_officeTerminationQueryListener;
				
				public UnoConnection (UnoObjectPointer <XComponentContext> a_originalRemoteUnoObjectsContext, String a_unoConnectionIdentification, XBridge a_unoBridge, InitialUnoObjectsProvider a_initialUnoObjectsProvider, List <UnoConnectionEventsListener> a_eventsListeners) {
					i_remoteUnoObjectsContext = new RemoteUnoObjectsContext (a_originalRemoteUnoObjectsContext, a_unoConnectionIdentification);
					i_remoteUnoObjectsContext.setUpForOfficeInstance ();
					i_unoBridge = (XComponent) a_unoBridge;
					i_initialUnoObjectsProvider = a_initialUnoObjectsProvider;
					i_eventsListeners = a_eventsListeners;
					i_unoBridge.addEventListener (this);
					if (i_eventsListeners != null) {
						foreach (UnoConnectionEventsListener l_eventsListener in i_eventsListeners){
							l_eventsListener.connected (this);
						}
					}
					// add local singletons Start
					i_remoteUnoObjectsContext.addLocalProperty (UnoObjectsContextPropertyNamesSet.c_unoConnectionSingleton_string, this);
					// add local singletons End
					UnoDesktop l_unoDesktop = (UnoDesktop) (i_remoteUnoObjectsContext.getLocalPropertyValue (UnoObjectsContextPropertyNamesSet.c_unoDesktopSingleton_string));
					if (l_unoDesktop != null) {
						i_officeTerminationQueryListener = new OfficeTerminationQueryListener (this);
						l_unoDesktop.getUnderlyingUnoObject ().getAddress <XDesktop> ().addTerminateListener (i_officeTerminationQueryListener);
					}
					else {
						i_officeTerminationQueryListener = null;
					}
				}
				
				~UnoConnection () {
				}
				
				public RemoteUnoObjectsContext getRemoteUnoObjectsContext () {
					return i_remoteUnoObjectsContext;
				}
				
				public void disconnect () {
					if (i_unoBridge != null) {
						try {
							if (i_officeTerminationQueryListener != null) {
								UnoDesktop l_unoDesktop = (UnoDesktop) (i_remoteUnoObjectsContext.getLocalPropertyValue (UnoObjectsContextPropertyNamesSet.c_unoDesktopSingleton_string));
								l_unoDesktop.getUnderlyingUnoObject ().getAddress <XDesktop> ().removeTerminateListener (i_officeTerminationQueryListener);
							}
							i_unoBridge.dispose ();
						}
						catch (System.Exception l_exception) {
							Publisher.logErrorInformation (String.Format ("### UnoConnection.disconnect error: {0:s}.", l_exception.ToString ()));
						}
					}
				}
				
				public void disposing (EventObject a_event) {
					if (i_eventsListeners != null) {
						foreach (UnoConnectionEventsListener l_eventsListener in i_eventsListeners) {
							l_eventsListener.disconnected (this);
						}
					}
					i_remoteUnoObjectsContext = null;
					i_unoBridge = null;
					i_initialUnoObjectsProvider = null;
					i_eventsListeners = null;
					i_officeTerminationQueryListener = null;
				}
			}
		}
	}
}

