namespace theBiasPlanet {
	namespace unoUtilities {
		namespace officeInstance {
			using System;
			using System.Collections.Generic;
			using System.Threading;
			using unoidl.com.sun.star.container;
			using unoidl.com.sun.star.frame;
			using unoidl.com.sun.star.lang;
			//using unoidl.com.sun.star.uno;
			using unoidl.com.sun.star.util;
			using theBiasPlanet.coreUtilities.constantsGroups;
			using theBiasPlanet.coreUtilities.messagingHandling;
			using unoidl.theBiasPlanet.unoDatumTypes.unoUtilities.officeInstance;
			using theBiasPlanet.unoUtilities.constantsGroups.propertyNamesSets;
			using theBiasPlanet.unoUtilities.displayElements;
			using theBiasPlanet.unoUtilities.documents;
			using theBiasPlanet.unoUtilities.pointers;
			using theBiasPlanet.unoUtilities.unoComponentBases;
			using theBiasPlanet.unoUtilities.unoDataHandling;
			using theBiasPlanet.unoUtilities.unoObjectsContexts;
			
			public class OfficeInstance: UnoComponentBase, XOfficeInstance {
				private RemoteUnoObjectsContext i_remoteUnoObjectsContext = null;
				private Object i_threadsCondition;
				private Int32 i_criticalSessionCount;
				private Boolean i_serverIsPlannedToBeTerminated;
				private UnoDesktop i_unoDesktop;
				
				public OfficeInstance (RemoteUnoObjectsContext a_remoteUnoObjectsContext) {
					i_remoteUnoObjectsContext = a_remoteUnoObjectsContext;
					i_criticalSessionCount = 0;
					i_serverIsPlannedToBeTerminated = false;
					i_threadsCondition = new Object ();
					i_unoDesktop = (UnoDesktop) (i_remoteUnoObjectsContext.getLocalPropertyValue (UnoObjectsContextPropertyNamesSet.c_unoDesktopSingleton_string));
					if (UnoDatumConverter.getObject (i_remoteUnoObjectsContext.getValueByName (UnoObjectsContextPropertyNamesSet.c_unoOfficeInstanceSingleton_string)) == null) {
						try {
							i_remoteUnoObjectsContext.addExtraGlobalProperty (UnoObjectsContextPropertyNamesSet.c_unoOfficeInstanceSingleton_string, UnoDatumConverter.getAny (this, typeof (XOfficeInstance)));
						}
						catch (ElementExistException) {
						}
					}
				}
				
				~OfficeInstance () {
				}
				
				public Boolean incrementCriticalSessionCount (String a_connectionIdentification) {
					lock (i_threadsCondition) {
						if (i_serverIsPlannedToBeTerminated) {
							return false;
						}
						else {
							i_criticalSessionCount ++;
							Publisher.logDebugInformation (String.Format ("### the critical sessions count has been incremented to '{0:d}'.", i_criticalSessionCount));
							return true;
						}
					}
				}
				
				public void decrementCriticalSessionCount (String a_connectionIdentification) {
					lock (i_threadsCondition) {
						i_criticalSessionCount --;
						Publisher.logDebugInformation (String.Format ("### the critical sessions count has been decremented to '{0:d}'.", i_criticalSessionCount));
						if (i_criticalSessionCount <= 0) {
							Monitor.PulseAll (i_threadsCondition);
						}
					}
				}
				
				public Int32 getCriticalSessionCount () {
					lock (i_threadsCondition) {
						return i_criticalSessionCount;
					}
				}
				
				public Boolean serverIsPlannedToBeTerminated () {
					lock (i_threadsCondition) {
						return i_serverIsPlannedToBeTerminated;
					}
				}
				
				public void setServerAsPlannedToBeTerminated () {
					lock (i_threadsCondition) {
						i_serverIsPlannedToBeTerminated = true;
					}
				}
				
				public Boolean waitUntilNoCriticalSession (Int32 a_timeOutInMilliseconds) {
					lock (i_threadsCondition) {
						if (i_criticalSessionCount > 0) {
							if (a_timeOutInMilliseconds == GeneralConstantsConstantsGroup.c_unspecifiedInteger) {
								Monitor.Wait (i_threadsCondition);
								return true;
							}
							else {
								return Monitor.Wait (i_threadsCondition, a_timeOutInMilliseconds);
							}
						}
						else {
							return true;
						}
					}
				}
				
				public virtual Boolean terminate (Int32 a_gracePeriodInMillisecondsBeforeForcing) {
					setServerAsPlannedToBeTerminated ();
					waitUntilNoCriticalSession (a_gracePeriodInMillisecondsBeforeForcing);
					List <UnoObjectPointer <XComponent>> l_unoObjects = i_unoDesktop.getChildObjects ();
					foreach (UnoObjectPointer <XComponent> l_unoObject in l_unoObjects) {
						if (l_unoObject.getAddress <XCloseable> () == null) {
							continue;
						}
						try {
							UnoDocument l_unoDocument = new UnoDocument (i_remoteUnoObjectsContext, new UnoObjectPointer <XModel> (l_unoObject));
							Publisher.logWarningInformation (String.Format ("The document '%s' is left opened.", l_unoDocument.getUrl ()));
							try {
								l_unoDocument.close ();
								Publisher.logWarningInformation (String.Format ("The document '%s' has been closed.", l_unoDocument.getUrl ()));
							}
							catch (CloseVetoException) {
								Publisher.logWarningInformation (String.Format ("The document '%s' has been vetoed to be closed.", l_unoDocument.getUrl ()));
								l_unoObject.getAddress ().dispose ();
								Publisher.logWarningInformation (String.Format ("The document '%s' has been disposed.", l_unoDocument.getUrl ()));
							}
							catch (DisposedException) {
								Publisher.logWarningInformation (String.Format ("The document '%s' has been disposed.", l_unoDocument.getUrl ()));
							}
						}
						catch (Exception l_exception) {
							Publisher.logErrorInformation (String.Format ("An error has occurred: %s.", l_exception.ToString ()));
						}
					}
					try {
						i_remoteUnoObjectsContext.removeExtraGlobalProperty (UnoObjectsContextPropertyNamesSet.c_unoOfficeInstanceSingleton_string);
					}
					catch (NoSuchElementException) {
					}
					catch (Exception) {
					}
					Boolean l_shutdownTryResult = false;
					// loops because the shutdown may be vetoed.
					try {
						while (true) {
							try {
								l_shutdownTryResult = i_unoDesktop.getUnderlyingUnoObject ().getAddress <XDesktop> ().terminate ();
							}
							catch (DisposedException) {
								return true;
							}
							if (l_shutdownTryResult) {
								return l_shutdownTryResult;
							}
							Thread.Sleep (1000);
						}
					}
					finally {
					}	
				}
			}
		}
	}
}

