namespace theBiasPlanet {
	namespace unoUtilities {
		namespace unoObjectsContexts {
			using System;
			using System.Collections.Generic;
			using uno;
			using unoidl.com.sun.star.container;
			using unoidl.com.sun.star.lang;
			using unoidl.com.sun.star.security;
			using unoidl.com.sun.star.uno;
			using unoidl.com.sun.star.xml.crypto;
			using theBiasPlanet.coreUtilities.collectionsHandling;
			using theBiasPlanet.coreUtilities.constantsGroups;
			using theBiasPlanet.coreUtilities.stringsHandling;
			using unoidl.theBiasPlanet.unoDatumTypes.unoUtilities.officeInstance;
			using theBiasPlanet.unoUtilities.constantsGroups;
			using theBiasPlanet.unoUtilities.constantsGroups.propertyNamesSets;
			using theBiasPlanet.unoUtilities.displayElements;
			using theBiasPlanet.unoUtilities.pointers;
			using theBiasPlanet.unoUtilities.unoDataHandling;
			
			public class RemoteUnoObjectsContext: UnoObjectsContext {
				private UnoObjectPointer <XComponentContext> i_underlyingUnoObject = null;
				private UnoObjectPointer <XMultiComponentFactory> i_unoGlobalServicesManager = null;
				private UnoObjectPointer <XOfficeInstance> i_officeInstance;
				private UnoObjectPointer <XSecurityEnvironment> i_gnupgUnoSecurityEnvironment = null;
				private UnoObjectPointer <XSecurityEnvironment> i_networkSecurityServicesUnoSecurityEnvironment = null;
				
				public RemoteUnoObjectsContext (UnoObjectPointer <XComponentContext> a_underlyingUnoObject, String a_unoConnectionIdentification): base (a_unoConnectionIdentification) {
					i_underlyingUnoObject = a_underlyingUnoObject;
					i_unoGlobalServicesManager = new UnoObjectPointer <XMultiComponentFactory> (i_underlyingUnoObject.getAddress ().getServiceManager ());
				}
				
				~RemoteUnoObjectsContext () {
				}
				
				public UnoObjectPointer <XComponentContext> getUnderlyingUnoObject () {
					return i_underlyingUnoObject;
				}
				
				override public Any getValueByName (String a_propertyName) {
					return i_underlyingUnoObject.getAddress ().getValueByName (a_propertyName);
				}
				
				override public void addExtraGlobalProperty (String a_propertyName, Any a_propertyValue) {
					i_underlyingUnoObject.getAddress <XNameContainer> ().insertByName (a_propertyName, a_propertyValue);
				}
				
				override public void removeExtraGlobalProperty (String a_propertyName) {
					i_underlyingUnoObject.getAddress <XNameContainer> ().removeByName  (a_propertyName);
				}
				
				override public XMultiComponentFactory getServiceManager () {
					return i_unoGlobalServicesManager.getAddress ();
				}
				
				public UnoObjectPointer <T> getUnoServiceInstance <T> (String a_unoServiceName, List <Object> a_unoServiceInstantiationArguments) where T: class {
					if (a_unoServiceInstantiationArguments == null) {
						return new UnoObjectPointer <T> (getServiceManager ().createInstanceWithContext (a_unoServiceName, this));
					}
					else {
						return new UnoObjectPointer <T> (getServiceManager ().createInstanceWithArgumentsAndContext (a_unoServiceName, ArraysFactory.createArray <Any> (a_unoServiceInstantiationArguments), this));
					}
				}
				
				public Boolean setUpForOfficeInstance () {
					try {
						UnoDesktop l_unoDesktop = UnoDesktop.getInstance (this);
						addLocalProperty (UnoObjectsContextPropertyNamesSet.c_unoDesktopSingleton_string, l_unoDesktop);
						if (i_officeInstance == null) {
							Object l_officeInstance = (Object) UnoDatumConverter.getObject (getValueByName (UnoObjectsContextPropertyNamesSet.c_unoOfficeInstanceSingleton_string));
							if (l_officeInstance != null) {
								i_officeInstance = new UnoObjectPointer <XOfficeInstance> (l_officeInstance);
							}
						}
						return true;
					}
					catch (System.Exception) {
						// the remote UNO environment may not be any Office instance, and it is OK.
					}
					return false;
				}
				
				public Boolean officeInstanceSingletonIsSet () {
					return i_officeInstance != null;
				}
				
				public Boolean incrementCriticalSessionCount (String a_connectionIdentification) {
					if (i_officeInstance != null) {
						return i_officeInstance.getAddress ().incrementCriticalSessionCount (a_connectionIdentification);
					}
					else {
						return false;
					}
				}
				
				public void decrementCriticalSessionCount (String a_connectionIdentification) {
					if (i_officeInstance != null) {
						i_officeInstance.getAddress ().decrementCriticalSessionCount (a_connectionIdentification);
					}
				}
				
				public int getCriticalSessionCount () {
					if (i_officeInstance != null) {
						return i_officeInstance.getAddress ().getCriticalSessionCount ();
					}
					else {
						return GeneralConstantsConstantsGroup.c_unspecifiedInteger;
					}
				}
				
				public Boolean serverIsPlannedToBeTerminated () {
					if (i_officeInstance != null) {
						return i_officeInstance.getAddress ().serverIsPlannedToBeTerminated ();
					}
					else {
						return false;
					}
				}
				
				private UnoObjectPointer <XCertificate> selectCertificateFromCertificates (List <UnoObjectPointer <XCertificate>> a_certificates, String a_certificateIssuerName, Byte [] a_certificateSerialNumber) {
					if (a_certificates != null) {
						foreach (UnoObjectPointer <XCertificate> l_certificate in a_certificates) {
							if (a_certificateIssuerName == l_certificate.getAddress ().IssuerName) {
								if (a_certificateSerialNumber == null) {
									return l_certificate;
								}
								else {
									Byte [] l_certificateSerialNumber = l_certificate.getAddress ().SerialNumber;
									if (a_certificateSerialNumber.Length == l_certificateSerialNumber.Length) {
										int l_certificateSerialNumberByteIndex = GeneralConstantsConstantsGroup.c_iterationStartNumber;
										foreach (Byte l_certificateSerialNumberByte in l_certificateSerialNumber) {
											if (a_certificateSerialNumber [l_certificateSerialNumberByteIndex] != l_certificateSerialNumberByte) {
												break;
											}
											l_certificateSerialNumberByteIndex ++;
										}
										if (l_certificateSerialNumberByteIndex == l_certificateSerialNumber.Length) {
											return l_certificate;
										}
									}
								}
							}
						}
					}
					return null;
				}
				
				public UnoObjectPointer <XCertificate> getGnupgCertificate (String a_certificateIssuerName) {
					if (i_gnupgUnoSecurityEnvironment == null) {
						UnoObjectPointer <XSEInitializer> l_gnupgSecurityEnvironmentInitializer = this.getUnoServiceInstance <XSEInitializer> (UnoServiceNamesConstantsGroup.c_com_sun_star_xml_crypto_GPGSEInitializer, null);
						UnoObjectPointer <XXMLSecurityContext> l_gnupgSecurityContext = new UnoObjectPointer <XXMLSecurityContext> (l_gnupgSecurityEnvironmentInitializer.getAddress ().createSecurityContext ("Gnupg")); 
						if (l_gnupgSecurityContext.getAddress ().getSecurityEnvironmentNumber () > 0) {
							i_gnupgUnoSecurityEnvironment = new UnoObjectPointer <XSecurityEnvironment> (l_gnupgSecurityContext.getAddress ().getSecurityEnvironmentByIndex (GeneralConstantsConstantsGroup.c_iterationStartNumber));
						}
					}
					if (! (i_gnupgUnoSecurityEnvironment.isEmpty ())) {
						List <UnoObjectPointer <XCertificate>> l_gnupgCertificates = UnoDatumConverter.getUnoObjects <XCertificate> (i_gnupgUnoSecurityEnvironment.getAddress().getAllCertificates ());
						return selectCertificateFromCertificates (l_gnupgCertificates, a_certificateIssuerName, null);
					}
					return null;
				}
				
				public UnoObjectPointer <XCertificate> getNetworkSecurityServicesCertificate (String a_certificateIssuerName, Byte [] a_certificateSerialNumber) {
					if (i_networkSecurityServicesUnoSecurityEnvironment == null) {
						UnoObjectPointer <XSEInitializer> l_networkSecurityServicesSecurityEnvironmentInitializer = this.getUnoServiceInstance <XSEInitializer> (UnoServiceNamesConstantsGroup.c_com_sun_star_xml_crypto_SEInitializer, null);
						UnoObjectPointer <XXMLSecurityContext> l_networkSecurityServicesSecurityContext = new UnoObjectPointer <XXMLSecurityContext> (l_networkSecurityServicesSecurityEnvironmentInitializer.getAddress ().createSecurityContext ("NetworkSecurityServices")); 
						if (l_networkSecurityServicesSecurityContext.getAddress().getSecurityEnvironmentNumber () > 0) {
							i_networkSecurityServicesUnoSecurityEnvironment = new UnoObjectPointer <XSecurityEnvironment> (l_networkSecurityServicesSecurityContext.getAddress ().getSecurityEnvironmentByIndex (GeneralConstantsConstantsGroup.c_iterationStartNumber));
						}
					}
					if (i_networkSecurityServicesUnoSecurityEnvironment != null) {
						List <UnoObjectPointer <XCertificate>> l_networkSecurityServicesCertificates = UnoDatumConverter.getUnoObjects <XCertificate> (i_networkSecurityServicesUnoSecurityEnvironment.getAddress ().getPersonalCertificates ());
						return selectCertificateFromCertificates (l_networkSecurityServicesCertificates, a_certificateIssuerName, a_certificateSerialNumber);
					}
					return null;
				}
			}
		}
	}
}

