from typing import cast
from typing import Dict
from typing import List
from typing import Optional
from abc import ABCMeta
from abc import abstractmethod
import uno
from uno import Any as UnoAny
from com.sun.star.bridge import XBridge;
from com.sun.star.bridge import XInstanceProvider
from com.sun.star.frame import TerminationVetoException
from com.sun.star.frame import XTerminateListener
from com.sun.star.lang import EventObject as com_sun_star_lang_EventObject
from com.sun.star.lang import XComponent
from com.sun.star.lang import XEventListener
from com.sun.star.uno import XComponentContext
from com.sun.star.uno import XInterface
from theBiasPlanet.coreUtilities.messagingHandling.Publisher import Publisher
from theBiasPlanet.unoUtilities.constantsGroups.UnoDefaultValuesConstantsGroup import UnoDefaultValuesConstantsGroup
from theBiasPlanet.unoUtilities.constantsGroups.propertyNamesSets.UnoObjectsContextPropertyNamesSet import UnoObjectsContextPropertyNamesSet
from theBiasPlanet.unoUtilities.displayElements.UnoDesktop import UnoDesktop
from theBiasPlanet.unoUtilities.pointers.UnoObjectPointer import UnoObjectPointer
from theBiasPlanet.unoUtilities.unoComponentBases.UnoComponentBase import UnoComponentBase
from theBiasPlanet.unoUtilities.unoDataHandling.UnoDatumConverter import UnoDatumConverter
from theBiasPlanet.unoUtilities.unoObjectsContexts.LocalUnoObjectsContext import LocalUnoObjectsContext
from theBiasPlanet.unoUtilities.unoObjectsContexts.RemoteUnoObjectsContext import RemoteUnoObjectsContext
from theBiasPlanet.unoUtilities.unoObjectsContexts.RemoteUnoObjectsContextInterface import RemoteUnoObjectsContextInterface

class UnoConnection (UnoComponentBase, XEventListener):
	class InitialUnoObjectsProvider (UnoComponentBase, XInstanceProvider):
		def __init__ (a_this: "UnoConnection.InitialUnoObjectsProvider", a_localUnoObjectsContext: "LocalUnoObjectsContext") -> None:
			a_this.i_localUnoObjectsContext: "LocalUnoObjectsContext" = None
			
			a_this.i_localUnoObjectsContext = a_localUnoObjectsContext
		
		def __del__ (a_this: "UnoConnection.InitialUnoObjectsProvider") -> None:
			None
		
		def getInstance (a_this: "UnoConnection.InitialUnoObjectsProvider", a_initialUnoObjectName: str) -> XInterface:
			if (UnoDefaultValuesConstantsGroup.c_initiallyOfferedUnoObjectName == a_initialUnoObjectName):
				return a_this.i_localUnoObjectsContext
			else:
				return None
	
	class UnoConnectionEventsListener (metaclass=ABCMeta):
		@abstractmethod
		def connected (a_this: "UnoConnection.UnoConnectionEventsListener", a_unoConnection: "UnoConnection") -> None:
			raise NotImplementedError ()
		
		@abstractmethod
		def disconnected (a_this: "UnoConnection.UnoConnectionEventsListener", a_unoConnection: "UnoConnection") -> None:
			raise NotImplementedError ()
	class OfficeTerminationQueryListener (UnoComponentBase, XTerminateListener):
		def __init__ (a_this: "UnoConnection.OfficeTerminationQueryListener", a_unoConnection: "UnoConnection") -> None:
			a_this.i_unoConnection: "UnoConnection"
			
			a_this.i_unoConnection = a_unoConnection
		
		def queryTermination (a_this: "UnoConnection.OfficeTerminationQueryListener", a_event: com_sun_star_lang_EventObject) -> None:
			Publisher.logNormalInformation ("### The termination of the office instance is queried.")
			l_remoteUnoObjectsContext: "RemoteUnoObjectsContextInterface" = a_this.i_unoConnection.getRemoteUnoObjectsContext ()
			if l_remoteUnoObjectsContext.officeInstanceSingletonIsSet ():
				if not (l_remoteUnoObjectsContext.serverIsPlannedToBeTerminated ()) or l_remoteUnoObjectsContext.getCriticalSessionCount () > 0:
					Publisher.logNormalInformation ("### And vetoed.")
					raise TerminationVetoException ()
				else:
					Publisher.logNormalInformation ("### And accepted.")
		
		def notifyTermination (a_this: "UnoConnection.OfficeTerminationQueryListener", a_event: com_sun_star_lang_EventObject) -> None:
			Publisher.logNormalInformation ("### The termination of the office instance is notified.")
			#a_this.i_unoConnection.disconnect ()
		
		def disposing (a_this: "UnoConnection.OfficeTerminationQueryListener", a_event: com_sun_star_lang_EventObject) -> None:
			None
	def __init__ (a_this: "UnoConnection", a_originalObjectsContext: "UnoObjectPointer [XComponentContext]", a_unoConnectionIdentification: str, a_unoBridge: XBridge, a_initialUnoObjectsProvider: "UnoConnection.InitialUnoObjectsProvider", a_eventsListeners: "Optional [List [UnoConnection.UnoConnectionEventsListener]]") -> None:
		a_this.i_remoteUnoObjectsContext: "Optional [RemoteUnoObjectsContextInterface]"
		a_this.i_unoBridge: XComponent
		a_this.i_initialUnoObjectsProvider: "UnoConnection.InitialUnoObjectsProvider"
		a_this.i_eventsListeners: "Optional [List [UnoConnection.UnoConnectionEventsListener]]"
		a_this.i_officeTerminationQueryListener: "Optional [UnoConnection.OfficeTerminationQueryListener]"
		
		a_this.i_remoteUnoObjectsContext = RemoteUnoObjectsContext (a_originalObjectsContext, a_unoConnectionIdentification)
		a_this.i_remoteUnoObjectsContext.setUpForOfficeInstance ()
		a_this.i_unoBridge = cast (XComponent, a_unoBridge)
		a_this.i_initialUnoObjectsProvider = a_initialUnoObjectsProvider
		a_this.i_eventsListeners = a_eventsListeners
		a_this.i_unoBridge.addEventListener (a_this)
		if (a_this.i_eventsListeners is not None):
			l_eventsListener: "UnoConnection.UnoConnectionEventsListener"
			for l_eventsListener in a_this.i_eventsListeners:
				l_eventsListener.connected (a_this)
		# add local singletons Start
		a_this.i_remoteUnoObjectsContext.addLocalProperty (UnoObjectsContextPropertyNamesSet.c_unoConnectionSingleton_string, a_this);
		# add local singletons End
		l_unoDesktop: "UnoDesktop" = cast (UnoDesktop, (a_this.i_remoteUnoObjectsContext.getLocalPropertyValue (UnoObjectsContextPropertyNamesSet.c_unoDesktopSingleton_string)))
		if l_unoDesktop is not None:
			a_this.i_officeTerminationQueryListener = UnoConnection.OfficeTerminationQueryListener (a_this)
			l_unoDesktop.getUnderlyingUnoObject ().getAddress ().addTerminateListener (a_this.i_officeTerminationQueryListener)
		else:
			a_this.i_officeTerminationQueryListener = None
	
	def __del__ (a_this: "UnoConnection") -> None:
		None
	
	def getRemoteUnoObjectsContext (a_this: "UnoConnection") -> "Optional [RemoteUnoObjectsContextInterface]":
		return a_this.i_remoteUnoObjectsContext
	
	def disconnect (a_this: "UnoConnection") -> None:
		if (not (a_this.i_unoBridge is None)):
			try:
				if a_this.i_officeTerminationQueryListener is not None:
					l_unoDesktop: "UnoDesktop" = cast (UnoDesktop, a_this.i_remoteUnoObjectsContext.getLocalPropertyValue (UnoObjectsContextPropertyNamesSet.c_unoDesktopSingleton_string))
					l_unoDesktop.getUnderlyingUnoObject ().getAddress ().removeTerminateListener (a_this.i_officeTerminationQueryListener)
				a_this.i_unoBridge.dispose ()
			except (Exception) as l_exception:
				Publisher.logErrorInformation ("### UnoConnection.disconnect error: {0:s}.".format (str (l_exception)))
	
	def disposing (a_this: "UnoConnection", a_event: com_sun_star_lang_EventObject) -> None:
		if (a_this.i_eventsListeners is not None):
			l_eventsListener: "UnoConnection.UnoConnectionEventsListener"
			for l_eventsListener in a_this.i_eventsListeners:
				l_eventsListener.disconnected (a_this)
		a_this.i_remoteUnoObjectsContext = None
		a_this.i_unoBridge = None
		a_this.i_initialUnoObjectsProvider = None
		a_this.i_eventsListeners = None
		a_this.i_officeTerminationQueryListener = None

