from typing import cast
from typing import List
from typing import Optional
from com.sun.star.beans import PropertyValue
from com.sun.star.beans import XPropertySet
from com.sun.star.container import XIndexAccess
from com.sun.star.container import XNamed
from com.sun.star.frame import XModel
from com.sun.star.frame import XStorable2
from com.sun.star.frame import XSynchronousDispatch
from com.sun.star.lang import XComponent
from com.sun.star.sheet import XSpreadsheetDocument
from com.sun.star.sheet import XSpreadsheets2
from com.sun.star.util import URL as com_sun_star_util_URL
from com.sun.star.util import XCloseable
from theBiasPlanet.coreUtilities.collectionsHandling.ListsFactory import ListsFactory
from theBiasPlanet.coreUtilities.constantsGroups.GeneralConstantsConstantsGroup import GeneralConstantsConstantsGroup
from theBiasPlanet.coreUtilities.messagingHandling.Publisher import Publisher
from theBiasPlanet.unoUtilities.constantsGroups.UnoDocumentStoringFilterNamesConstantsGroup import UnoDocumentStoringFilterNamesConstantsGroup 
from theBiasPlanet.unoUtilities.constantsGroups.propertyNamesSets.UnoDocumentOpeningEnumerablePropertyNamesSet import UnoDocumentOpeningEnumerablePropertyNamesSet
from theBiasPlanet.unoUtilities.constantsGroups.propertyNamesSets.UnoDocumentStoringEnumerablePropertyNamesSet import UnoDocumentStoringEnumerablePropertyNamesSet
from theBiasPlanet.unoUtilities.constantsGroups.propertyNamesSets.UnoSpreadSheetPropertyNamesSet import UnoSpreadSheetPropertyNamesSet
from theBiasPlanet.unoUtilities.displayElements.UnoDesktop import UnoDesktop
from theBiasPlanet.unoUtilities.documentsHandling.UnoDocumentTailor import UnoDocumentTailor
from theBiasPlanet.unoUtilities.pointers.UnoObjectPointer import UnoObjectPointer
from theBiasPlanet.unoUtilities.propertiesHandling.UnoPropertiesHandler import UnoPropertiesHandler
from theBiasPlanet.unoUtilities.unoDataHandling.UnoDatumConverter import UnoDatumConverter
from theBiasPlanet.unoUtilities.unoObjectsContexts.RemoteUnoObjectsContextInterface import RemoteUnoObjectsContextInterface

class FilesConverter:
	c_csvFormatStringFormat: str = "{0:d},{1:d},{2:d},1,,0,{3:s},false,{4:s},{5:s},false" # the CSV items delimiter character code (can be specified like 'Character.codePointAt ("\t", 0)'), the CSV text items quotation character code (can be specified like 'Character.codePointAt ("\"", 0)'), the encoding code (76 -> UTF-8, 65535 -> UCS-2, 65534 -> UCS-4, 11 -> US-ASCII, 69 -> EUC_JP, 64 -> SHIFT_JIS), whether all the text items are quoted, whether the contents are exported as shown, whether the formula themselves are exported
	
	def __init__ (a_this: "FilesConverter", a_unoDesktop: UnoDesktop) -> None:
		a_this.i_remoteUnoObjectsContext: "RemoteUnoObjectsContextInterface"
		a_this.i_fileOpeningUnoDispatcher: "UnoObjectPointer [XSynchronousDispatch]"
		
		a_this.i_remoteUnoObjectsContext = a_unoDesktop.getRemoteUnoObjectsContext ()
		a_this.i_fileOpeningUnoDispatcher = a_unoDesktop.getFileOpeningDispatcher ()
	
	def getRemoteUnoObjectsContext (a_this: "FilesConverter") -> "RemoteUnoObjectsContextInterface":
		return a_this.i_remoteUnoObjectsContext
	
	def convertFile (a_this: "FilesConverter", a_originalFileUrl: str, a_originalFilePassword: Optional [str], a_targetFileUrl: str, a_unoDocumentStoringProperties: Optional [List [PropertyValue]], a_unoDocumentTailors: "Optional [List [UnoDocumentTailor]]") -> bool:
		l_originalFileUrlInURL: com_sun_star_util_URL = UnoDatumConverter.getUrlInURL (a_originalFileUrl)
		l_unoDocumentOpeningProperties: Optional [List [PropertyValue]] = UnoPropertiesHandler.buildProperties (UnoDocumentOpeningEnumerablePropertyNamesSet.c_instance.getValues (), ListsFactory.createList (object, True, True, True, True, a_originalFilePassword if not (a_originalFilePassword is None) else ""))
		l_originalUnoDocument: "UnoObjectPointer [XModel]" = UnoObjectPointer (XModel, a_this.i_fileOpeningUnoDispatcher.getAddress ().dispatchWithReturnValue (l_originalFileUrlInURL, l_unoDocumentOpeningProperties))
		l_hasSucceeded: bool = False
		if not (l_originalUnoDocument.isEmpty ()):
			try:
				if a_unoDocumentTailors is not None:
					l_unoDocumentTailor: "UnoDocumentTailor"
					for l_unoDocumentTailor in a_unoDocumentTailors:
						l_unoDocumentTailor.tailor (l_originalUnoDocument)
				l_originalUnoDocument.getAddress ().storeToURL (a_targetFileUrl, a_unoDocumentStoringProperties)
				l_hasSucceeded = True
			except (Exception) as l_exception:
				raise l_exception
			l_originalUnoDocument.getAddress ().close (False)
		else:
			Publisher.logErrorInformation ("### The original file: '{0:s}' cannot be opened.".format (a_originalFileUrl))
		return l_hasSucceeded
	
	@staticmethod
	def createToCsvFileDocumentStoringProperties (a_itemsDelimiterCharacterCode: int, a_textItemQuotationCharacterCode: int, a_charactersEncodingCode: int, a_whetherAllTextItemsAreQuoted: bool, a_whetherContentsAreExportedAsShown: bool, a_whetherFormulaThemselvesAreExported: bool) -> Optional [List [PropertyValue]]:
		l_unoDocumentStoringPropertyNames: List [str] = ListsFactory.createList (
			str,
			UnoDocumentStoringEnumerablePropertyNamesSet.c_filterName_string,
			UnoDocumentStoringEnumerablePropertyNamesSet.c_filterData_string,
			UnoDocumentStoringEnumerablePropertyNamesSet.c_overwrites_boolean
		)
		l_unoDocumentStoringPropertyValues: List [object] = ListsFactory.createList (
			object,
			UnoDocumentStoringFilterNamesConstantsGroup.c_toCsvFileFilterName,
			FilesConverter.c_csvFormatStringFormat.format (a_itemsDelimiterCharacterCode, a_textItemQuotationCharacterCode, a_charactersEncodingCode, str (a_whetherAllTextItemsAreQuoted).lower (), str (a_whetherContentsAreExportedAsShown).lower (), str (a_whetherFormulaThemselvesAreExported).lower ()),
			True
		)
		return UnoPropertiesHandler.buildProperties (l_unoDocumentStoringPropertyNames, l_unoDocumentStoringPropertyValues)
	
	# a_targetFileNamingRule: 0 -> augmented by the sheet index, 1 -> augmented by the sheet name
	def convertSpreadSheetsDocumentFileToCsvFiles (a_this: "FilesConverter", a_originalFileUrl: str, a_originalFilePassword: Optional [str], a_targetFileUrlBase: str, a_unoDocumentStoringProperties: Optional [List [PropertyValue]], a_unoDocumentTailors: "Optional [List [UnoDocumentTailor]]", a_targetFileNamingRule: int, a_hiddenSpreadSheetsAreWritten: bool) -> bool:
		l_originalFileUrlInURL: com_sun_star_util_URL = UnoDatumConverter.getUrlInURL (a_originalFileUrl)
		l_unoDocumentOpeningProperties: Optional [List [PropertyValue]] = UnoPropertiesHandler.buildProperties (UnoDocumentOpeningEnumerablePropertyNamesSet.c_instance.getValues (), ListsFactory.createList (object, True, True, True, True, a_originalFilePassword if not (a_originalFilePassword is None) else ""))
		l_originalUnoDocument: "UnoObjectPointer [XModel]" = UnoObjectPointer (XModel, a_this.i_fileOpeningUnoDispatcher.getAddress ().dispatchWithReturnValue (l_originalFileUrlInURL, l_unoDocumentOpeningProperties))
		l_hasSucceeded: bool = False
		if not (l_originalUnoDocument.isEmpty ()):
			try:
				if a_unoDocumentTailors is not None:
					l_unoDocumentTailor: "UnoDocumentTailor"
					for l_unoDocumentTailor in a_unoDocumentTailors:
						l_unoDocumentTailor.tailor (l_originalUnoDocument)
				# Is not really checking whether the document is a spread sheets document
				l_spreadSheets: "UnoObjectPointer [XSpreadsheets2]" = UnoObjectPointer (XSpreadsheets2, l_originalUnoDocument.getAddress ().getSheets ())
				l_numberOfSheets: int = l_spreadSheets.getAddress ().getCount ()
				l_spreadSheet: "UnoObjectPointer [XNamed]" = None
				l_targetFileNameAugmentation: Optional [str] = None
				l_lastPeriodPositionInTargetFileUrlBase: int = GeneralConstantsConstantsGroup.c_unspecifiedInteger
				try:
					l_lastPeriodPositionInTargetFileUrlBase = a_targetFileUrlBase.rindex (".")
				except (Exception) as l_exception:
					None
				l_augmentedTargetFileUrl: Optional [str] = None
				l_whetherSheetIsVisible: bool = False
				l_sheetIndex: int
				for l_sheetIndex in range (GeneralConstantsConstantsGroup.c_iterationStartNumber, l_numberOfSheets, 1):
					l_spreadSheet = UnoObjectPointer (XNamed, l_spreadSheets.getAddress ().getByIndex (l_sheetIndex))
					if a_hiddenSpreadSheetsAreWritten:
						None
					else:
						l_whetherSheetIsVisible = l_spreadSheet.getAddress ().getPropertyValue (UnoSpreadSheetPropertyNamesSet.c_whetherIsVisible_boolean) 
						if not l_whetherSheetIsVisible:
							continue
					if a_targetFileNamingRule == 0:
						l_targetFileNameAugmentation = "{0:d}".format (l_sheetIndex)
					else:
						l_targetFileNameAugmentation = l_spreadSheet.getAddress ().getName ()
					if l_lastPeriodPositionInTargetFileUrlBase >= GeneralConstantsConstantsGroup.c_iterationStartNumber:
						l_augmentedTargetFileUrl = "{0:s}_{1:s}{2:s}".format (a_targetFileUrlBase [GeneralConstantsConstantsGroup.c_iterationStartNumber: l_lastPeriodPositionInTargetFileUrlBase], l_targetFileNameAugmentation, a_targetFileUrlBase [l_lastPeriodPositionInTargetFileUrlBase:])
					else:
						l_augmentedTargetFileUrl = "{0:s}_{1:s}".format (a_targetFileUrlBase, l_targetFileNameAugmentation)
					if l_sheetIndex != GeneralConstantsConstantsGroup.c_iterationStartNumber:
						l_spreadSheets.getAddress ().moveByName (l_spreadSheet.getAddress ().getName (), GeneralConstantsConstantsGroup.c_iterationStartNumber)
					l_originalUnoDocument.getAddress ().storeToURL (l_augmentedTargetFileUrl, a_unoDocumentStoringProperties)
				l_hasSucceeded = True
			except (Exception) as l_exception:
				raise l_exception
			l_originalUnoDocument.getAddress ().close (False)
		else:
			Publisher.logErrorInformation ("### The original file: '{0:s}' cannot be opened.".format (a_originalFileUrl))
		return l_hasSucceeded

