from typing import Union
from typing import cast
from importlib.abc import SourceLoader
import uno
from uno import ByteSequence
from com.sun.star.io import XInputStream
from com.sun.star.ucb import XSimpleFileAccess
from com.sun.star.uno import XComponentContext
from theBiasPlanet.coreUtilities.constantsGroups.DefaultValuesConstantsGroup import DefaultValuesConstantsGroup
from theBiasPlanet.coreUtilities.constantsGroups.FileNameSuffixesConstantsGroup import FileNameSuffixesConstantsGroup 
from theBiasPlanet.coreUtilities.constantsGroups.GeneralConstantsConstantsGroup import GeneralConstantsConstantsGroup 
from theBiasPlanet.coreUtilities.constantsGroups.InputPropertiesConstantsGroup import InputPropertiesConstantsGroup
from theBiasPlanet.unoUtilities.constantsGroups.UnoServiceNamesConstantsGroup import UnoServiceNamesConstantsGroup 
from theBiasPlanet.unoUtilities.pointers.UnoObjectPointer import UnoObjectPointer
from theBiasPlanet.unoUtilities.unoObjectsContexts.RemoteUnoObjectsContextInterface import RemoteUnoObjectsContextInterface 

class UnoExtendedPythonSourceLoader (SourceLoader):
	c_readingBlockSize: int = DefaultValuesConstantsGroup.c_smallBufferSize
	
	def __init__ (a_this: "UnoExtendedPythonSourceLoader", a_remoteUnoObjectsContext: "RemoteUnoObjectsContextInterface", a_uriPrefix: str) -> None:
		a_this.i_simpleFilesAccessUnoService: "UnoObjectPointer [XSimpleFileAccess]"
		a_this.i_uriPrefix: str
		
		a_this.i_uriPrefix = a_uriPrefix
		
		a_this.i_simpleFilesAccessUnoService = a_remoteUnoObjectsContext.getUnoServiceInstance (XSimpleFileAccess, UnoServiceNamesConstantsGroup.c_com_sun_star_ucb_SimpleFileAccess, None)
	
	def get_filename (a_this: "UnoExtendedPythonSourceLoader", a_moduleName: str) -> str:
		return "{0:s}{1:s}.{2:s}".format (a_this.i_uriPrefix, a_moduleName.replace (GeneralConstantsConstantsGroup.c_pythonPackagesDelimiter, GeneralConstantsConstantsGroup.c_urlDirectoriesDelimiter), FileNameSuffixesConstantsGroup.c_pythonModuleFileNameSuffix)
	
	def get_data (a_this: "UnoExtendedPythonSourceLoader", a_url: Union [bytes, str]) -> bytes:
		try:
			l_inputStream: "UnoObjectPointer [XInputStream]" = UnoObjectPointer (XInputStream, a_this.i_simpleFilesAccessUnoService.getAddress ().openFileRead (a_url))
			l_readLength: int = InputPropertiesConstantsGroup.c_noDataRetrieved
			l_inputDatum: ByteSequence = ByteSequence (b"")
			l_inputBuffer: ByteSequence = None
			while True:
				l_readLength, l_inputBuffer = l_inputStream.getAddress ().readBytes (None, DefaultValuesConstantsGroup.c_smallBufferSize)
				l_inputDatum = l_inputDatum + l_inputBuffer
				if l_readLength < DefaultValuesConstantsGroup.c_smallBufferSize:
					break
		finally:
			if l_inputStream is not None:
				l_inputStream.getAddress ().closeInput ()
		return l_inputDatum.value

