from typing import List
from typing import Match
from typing import Optional
from typing import Pattern
from typing import Type
from typing import TypeVar
from typing import cast
import re
from com.sun.star.uno import XInterface
from com.sun.star.util import URL as com_sun_star_util_URL
from uno import Any as UnoAny
from theBiasPlanet.coreUtilities.stringsHandling.StringHandler import StringHandler
from theBiasPlanet.unoUtilities.constantsGroups.UnoDatumTypeNamesConstantsGroup import UnoDatumTypeNamesConstantsGroup
from theBiasPlanet.unoUtilities.constantsGroups.UnoRegularExpressionsConstantsGroup import UnoRegularExpressionsConstantsGroup
from theBiasPlanet.unoUtilities.pointers.UnoObjectPointer import UnoObjectPointer

T = TypeVar ("T", bound=XInterface)

class UnoDatumConverter:
	@staticmethod
	def getObject (a_originalObject: Optional [object]) -> Optional [object]:
		if a_originalObject is None:
			return None
		else:
			if isinstance (a_originalObject, UnoAny):
				return cast (UnoAny, a_originalObject).value
			else:
				return a_originalObject
	
	"""
	a_typeString: Use theBiasPlanet.unoUtilities.constantsGroups.UnoDatumTypeNamesConstantsGroup
	"""
	@staticmethod
	def getAny (a_originalObject: Optional [object], a_typeString: Optional [str] = None) -> Optional [UnoAny]:
		if a_originalObject is None:
			return None
		else:
			if isinstance (a_originalObject, UnoAny):
				return a_originalObject
			else:
				l_type: type = type (a_originalObject)
				l_typeString: str
				if a_typeString is None:
					if l_type == str:
						l_typeString = UnoDatumTypeNamesConstantsGroup.c_stringTypeName
					elif l_type == int:
						l_typeString = UnoDatumTypeNamesConstantsGroup.c_longTypeName
					elif l_type == bool:
						l_typeString = UnoDatumTypeNamesConstantsGroup.c_booleanTypeName
					elif l_type == float:
						l_typeString = UnoDatumTypeNamesConstantsGroup.c_floatTypeName
					elif l_type == list:
						l_typeString = "{0:s}{1:s}".format (UnoDatumTypeNamesConstantsGroup.c_sequenceTypeNamePrefix, UnoDatumTypeNamesConstantsGroup.c_anyTypeName)
					else:
						raise Exception ("The UNO datum type for '{0:s}' isn't implemented yet.".format (str (l_type)))
				else:
					l_typeString = a_typeString
				return UnoAny (l_typeString, a_originalObject)
	
	@staticmethod
	def getUnoObjects (a_type0: Type [T], a_unoProxies: List [T]) -> List ["UnoObjectPointer [T]"]:
		if a_unoProxies is None:
			return None
		l_unoObjects: "List [UnoObjectPointer [T]]" = []
		l_unoProxy: T
		for l_unoProxy in a_unoProxies:
			l_unoObjects.append (UnoObjectPointer (a_type0, l_unoProxy))
		return l_unoObjects
	
	@staticmethod
	def getUrlInURL (a_url: str) -> com_sun_star_util_URL:
		l_urlInURL: com_sun_star_util_URL = com_sun_star_util_URL ()
		l_urlInURL.Complete = a_url
		l_urlInURL.Main = l_urlInURL.Complete
		l_regularExpressionMatch: Optional [Match] = UnoRegularExpressionsConstantsGroup.c_urlRegularExpression.match (l_urlInURL.Complete, 0)
		if l_regularExpressionMatch is not None:
			l_urlInURL.Protocol = l_regularExpressionMatch.groups () [0]
			l_urlInURL.User = ""
			l_urlInURL.Password = ""
			l_urlInURL.Server = ""
			l_urlInURL.Port = 0
			l_urlInURL.Path = l_regularExpressionMatch.groups () [1]
			l_urlInURL.Name = ""
			l_urlInURL.Arguments = ""
			l_urlInURL.Mark = ""
		return l_urlInURL

