from typing import Any
from typing import Dict
from typing import List
from typing import Optional
from typing import Type
from typing import TypeVar
from typing import cast
import uno
from com.sun.star.lang import XMultiComponentFactory
from com.sun.star.security import XCertificate
from com.sun.star.uno import XComponentContext
from com.sun.star.uno import XInterface
from com.sun.star.xml.crypto import XSEInitializer
from com.sun.star.xml.crypto import XSecurityEnvironment
from com.sun.star.xml.crypto import XXMLSecurityContext
from theBiasPlanet.coreUtilities.constantsGroups.GeneralConstantsConstantsGroup import GeneralConstantsConstantsGroup
from theBiasPlanet.unoUtilities.constantsGroups.UnoServiceNamesConstantsGroup import UnoServiceNamesConstantsGroup
from theBiasPlanet.unoUtilities.constantsGroups.propertyNamesSets.UnoObjectsContextPropertyNamesSet import UnoObjectsContextPropertyNamesSet
from theBiasPlanet.unoUtilities.displayElements.UnoDesktop import UnoDesktop
from theBiasPlanet.unoUtilities.pointers.UnoObjectPointer import UnoObjectPointer
from theBiasPlanet.unoUtilities.unoDataHandling.UnoDatumConverter import UnoDatumConverter
from theBiasPlanet.unoUtilities.unoObjectsContexts.RemoteUnoObjectsContextInterface import RemoteUnoObjectsContextInterface
XOfficeInstance: Any = uno.getClass ("theBiasPlanet.unoDatumTypes.unoUtilities.officeInstance.XOfficeInstance")

T = TypeVar ("T", bound="XInterface")

class RemoteUnoObjectsContext (RemoteUnoObjectsContextInterface):
	def __init__ (a_this: "RemoteUnoObjectsContext", a_underlyingUnoObject: "UnoObjectPointer [XComponentContext]", a_unoConnectionIdentification: str) -> None:
		a_this.i_underlyingUnoObject: "UnoObjectPointer [XComponentContext]"
		a_this.i_unoGlobalServicesManager: "UnoObjectPointer [XMultiComponentFactory]"
		a_this.i_officeInstance: "UnoObjectPointer [XOfficeInstance]"
		a_this.i_gnupgUnoSecurityEnvironment: "UnoObjectPointer [XSecurityEnvironment]"
		a_this.i_networkSecurityServicesUnoSecurityEnvironment: "UnoObjectPointer [XSecurityEnvironment]"
		
		super ().__init__ (a_underlyingUnoObject, a_unoConnectionIdentification)
		a_this.i_underlyingUnoObject = a_underlyingUnoObject
		a_this.i_unoGlobalServicesManager = UnoObjectPointer (XMultiComponentFactory, a_this.i_underlyingUnoObject.getAddress ().getServiceManager ())
		a_this.i_officeInstance = None
	
	def __del__ (a_this: "RemoteUnoObjectsContext") -> None:
		None
	
	def getUnderlyingUnoObject (a_this: "RemoteUnoObjectsContext") -> "UnoObjectPointer [XComponentContext]":
		return a_this.i_underlyingUnoObject
	
	def getValueByName (a_this: "RemoteUnoObjectsContext", a_propertyName: str) -> object:
		return a_this.i_underlyingUnoObject.getAddress ().getValueByName (a_propertyName)
	
	def addExtraGlobalProperty (a_this: "RemoteUnoObjectsContext", a_propertyName: str, a_propertyValue: object) -> None:
		a_this.i_underlyingUnoObject.getAddress ().insertByName (a_propertyName, a_propertyValue)
	
	def removeExtraGlobalProperty (a_this: "RemoteUnoObjectsContext", a_propertyName: str) -> None:
		a_this.i_underlyingUnoObject.getAddress ().removeByName  (a_propertyName)
	
	def getServiceManager (a_this: "RemoteUnoObjectsContext") -> XMultiComponentFactory:
		return a_this.i_underlyingUnoObject.getAddress ().getServiceManager ()
	
	def getUnoServiceInstance (a_this: "RemoteUnoObjectsContext", a_type0: Type [T], a_unoServiceName: str, a_unoServiceInstantiationArguments: Optional [List [object]]) -> "UnoObjectPointer [T]":
		if a_type0 is None:
			return None
		if a_unoServiceInstantiationArguments is None:
			return UnoObjectPointer (a_type0, a_this.getServiceManager ().createInstanceWithContext (a_unoServiceName, a_this))
		else:
			return UnoObjectPointer (a_type0, a_this.getServiceManager ().createInstanceWithArgumentsAndContext (a_unoServiceName, a_unoServiceInstantiationArguments, a_this))
	
	def setUpForOfficeInstance (a_this: "RemoteUnoObjectsContext") -> bool:
		try:
			l_unoDesktop: "UnoDesktop" = UnoDesktop.getInstance (a_this)
			a_this.addLocalProperty (UnoObjectsContextPropertyNamesSet.c_unoDesktopSingleton_string, l_unoDesktop)
			if a_this.i_officeInstance is None:
				l_officeInstance: XInterface = cast (XInterface, UnoDatumConverter.getObject (a_this.getValueByName (UnoObjectsContextPropertyNamesSet.c_unoOfficeInstanceSingleton_string)))
				if l_officeInstance is not None:
					a_this.i_officeInstance = UnoObjectPointer (XOfficeInstance, l_officeInstance)
			return True
		except (Exception) as l_exception:
			# the remote UNO environment may not be any Office instance, and it is OK.
			None
		return False
	
	def officeInstanceSingletonIsSet (a_this: "RemoteUnoObjectsContext") -> bool:
		return a_this.i_officeInstance is not None
	
	def incrementCriticalSessionCount (a_this: "RemoteUnoObjectsContext", a_connectionIdentification: str) -> bool:
		if a_this.i_officeInstance is not None:
			return a_this.i_officeInstance.getAddress ().incrementCriticalSessionCount (a_connectionIdentification)
		else:
			return False
	
	def decrementCriticalSessionCount (a_this: "RemoteUnoObjectsContext", a_connectionIdentification: str) -> None:
		if a_this.i_officeInstance is not None:
			a_this.i_officeInstance.getAddress ().decrementCriticalSessionCount (a_connectionIdentification)
	
	def getCriticalSessionCount (a_this: "RemoteUnoObjectsContext") -> int:
		if a_this.i_officeInstance is not None:
			return a_this.i_officeInstance.getAddress ().getCriticalSessionCount ()
		else:
			return GeneralConstantsConstantsGroup.c_unspecifiedInteger
	
	def serverIsPlannedToBeTerminated (a_this: "RemoteUnoObjectsContext") -> bool:
		if a_this.i_officeInstance is not None:
			return a_this.i_officeInstance.getAddress ().serverIsPlannedToBeTerminated ()
		else:
			return False
	
	def selectCertificateFromCertificates (a_this: "RemoteUnoObjectsContext", a_certificates: List ["UnoObjectPointer [XCertificate]"], a_certificateIssuerName: str, a_certificateSerialNumber: List [int]) -> "UnoObjectPointer [XCertificate]":
		if a_certificates is not None:
			l_certificate: "UnoObjectPointer [XCertificate]" = None
			for l_certificate in a_certificates:
				if a_certificateIssuerName == l_certificate.getAddress ().IssuerName:
					if a_certificateSerialNumber is None:
						return l_certificate
					else:
						l_certificateSerialNumber: List [int] = l_certificate.getAddress ().getSerialNumber ()
						if len (a_certificateSerialNumber) == len (l_certificateSerialNumber):
							l_certificateSerialNumberByteIndex: int = GeneralConstantsConstantsGroup.c_iterationStartNumber
							l_certificateSerialNumberByte: int
							for l_certificateSerialNumberByte in l_certificateSerialNumber:
								if a_certificateSerialNumber [l_certificateSerialNumberByteIndex] != l_certificateSerialNumberByte:
									break
								l_certificateSerialNumberByteIndex = l_certificateSerialNumberByteIndex + 1
							if l_certificateSerialNumberByteIndex == len (l_certificateSerialNumber):
								return l_certificate
		return None
	
	def getGnupgCertificate (a_this: "RemoteUnoObjectsContext", a_certificateIssuerName: str) -> "UnoObjectPointer [XCertificate]":
		if a_this.i_gnupgUnoSecurityEnvironment is None:
			l_gnupgSecurityEnvironmentInitializer: "UnoObjectPointer [XSEInitializer]" = a_this.getUnoServiceInstance (XSEInitializer, UnoServiceNamesConstantsGroup.c_com_sun_star_xml_crypto_GPGSEInitializer, None)
			l_gnupgSecurityContext: "UnoObjectPointer [XXMLSecurityContext]" = UnoObjectPointer (XXMLSecurityContext, l_gnupgSecurityEnvironmentInitializer.getAddress ().createSecurityContext ("Gnupg"))
			if l_gnupgSecurityContext.getAddress ().getSecurityEnvironmentNumber () > 0:
				a_this.i_gnupgUnoSecurityEnvironment = UnoObjectPointer (XSecurityEnvironment, l_gnupgSecurityContext.getAddress ().getSecurityEnvironmentByIndex (GeneralConstantsConstantsGroup.c_iterationStartNumber))
		if not a_this.i_gnupgUnoSecurityEnvironment.isEmpty ():
			l_gnupgCertificates: List ["UnoObjectPointer [XCertificate]"]  = UnoDatumConverter.getUnoObjects (XCertificate, a_this.i_gnupgUnoSecurityEnvironment.getAddress ().getAllCertificates ())
			return a_this.selectCertificateFromCertificates (l_gnupgCertificates, a_certificateIssuerName, None)
		return None
	
	def getNetworkSecurityServicesCertificate (a_this: "RemoteUnoObjectsContext", a_certificateIssuerName: str, a_certificateSerialNumber: List [int]) -> "UnoObjectPointer [XCertificate]":
		if a_this.i_networkSecurityServicesUnoSecurityEnvironment is None:
			l_networkSecurityServicesSecurityEnvironmentInitializer: "UnoObjectPointer [XSEInitializer]" = a_this.getUnoServiceInstance (XSEInitializer, UnoServiceNamesConstantsGroup.c_com_sun_star_xml_crypto_SEInitializer, None)
			l_networkSecurityServicesSecurityContext: "UnoObjectPointer [XXMLSecurityContext]" = UnoObjectPointer (XXMLSecurityContext, l_networkSecurityServicesSecurityEnvironmentInitializer.getAddress ().createSecurityContext ("NetworkSecurityServices"))
			if l_networkSecurityServicesSecurityContext.getAddress ().getSecurityEnvironmentNumber () > 0:
				a_this.i_networkSecurityServicesUnoSecurityEnvironment = UnoObjectPointer (XSecurityEnvironment, l_networkSecurityServicesSecurityContext.getAddress ().getSecurityEnvironmentByIndex (GeneralConstantsConstantsGroup.c_iterationStartNumber))
		if a_this.i_networkSecurityServicesUnoSecurityEnvironment is not None:
			l_networkSecurityServicesCertificates: List ["UnoObjectPointer [XCertificate]"] = UnoDatumConverter.getUnoObjects (XCertificate, a_this.i_networkSecurityServicesUnoSecurityEnvironment.getAddress ().getPersonalCertificates ())
			return a_this.selectCertificateFromCertificates (l_networkSecurityServicesCertificates, a_certificateIssuerName, a_certificateSerialNumber)
		return None

